package ankistream;
import java.io.File;
import java.util.Random;
import java.awt.Toolkit;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane; 
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.Timer;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** A dialog for adding cards to the current card set and ccontainer.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class AddCardsDialog extends JDialog {

	// The preferences settings indicated on this form.
	AnkiStreamFrame owner;
	private CardContainer ccontainer;
	private CardSet cset; 
	private CardContainerDisplayPanel ccdisplaypanel;
	private MetaCard nextmetacard;
	
	// Controls
	MetaCardDisplayPanel mcdpanel;
	JPanel controlpanel;
	JButton addandcontinuebutton;
	JButton addandclosebutton;
	JButton cancelbutton;
	JLabel messagelabel;
	
	/** Creates and shows a new Edit Preferences dialog using the specified
	  * preferences.
	 */
	public AddCardsDialog(	AnkiStreamFrame newowner, 
							CardContainer newcc, 
							CardSet newcset,
							CardContainerDisplayPanel newccdisplaypanel ) {
		// Set the owner.
		super( newowner );
		
		owner = newowner;
		ccontainer = newcc;
		cset = newcset;
		ccdisplaypanel = newccdisplaypanel;

		// Layout Controls.
		getContentPane().setLayout( new BorderLayout() );

		// Instructions labels.
		JPanel p = new JPanel( new GridLayout(0,1) );
		JPanel p2 = new JPanel( new FlowLayout(FlowLayout.CENTER,0,0) );
		messagelabel = new JLabel( Strings.get(160) );
		p2.add( messagelabel );
		p.add( p2 );
		p2 = new JPanel( new FlowLayout(FlowLayout.CENTER,0,0) );
		messagelabel = 
			new JLabel( Strings.get(161) + 
					owner.getCardContainerTypeLowerCase() + Strings.get(162) );
		p2.add( messagelabel );
		p.add( p2 );
		p.setBorder( BorderFactory.createTitledBorder( "" ));
		getContentPane().add( p, BorderLayout.NORTH );

		// Action buttons.
		controlpanel = new JPanel( new GridLayout(0,1) );
		p = new JPanel( new FlowLayout() );
		addandcontinuebutton = new JButton( Strings.get(163) );
		addandcontinuebutton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { onAddContinue( ); }
		} );
		p.add( addandcontinuebutton );
		addandclosebutton = new JButton( Strings.get(164) );
		addandclosebutton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { onAddClose( ); }
		} );
		p.add( addandclosebutton );
		cancelbutton = new JButton( Strings.get(73) );
		cancelbutton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { dispose( ); }
		} );
		p.add( cancelbutton );
		controlpanel.add( p );
		controlpanel.add( p );
		controlpanel.setBorder( BorderFactory.createTitledBorder( "" ));
		getContentPane().add( controlpanel, BorderLayout.SOUTH );
		freshCard();
		
		setTitle( Strings.get(59) );
		pack();
		setLocationRelativeTo( owner );
		setModal( true );
		setVisible( true );
	}
	
	// Prepares a new card to be added to the card set.
	private void freshCard() {
		// TODO: We should be able to add other types of cards too besides
		// basic ones.
		nextmetacard = new BasicCard( cset.getNextAvailableID( ), "", "" );
		if ( mcdpanel != null ) {
			getContentPane().remove( mcdpanel );
		}
		// TODO: this is a temporary workaround for getting font sizes to
		// editpanel without FontController
		mcdpanel = nextmetacard.getEditPanel(
				(Component)addandcontinuebutton, 
				(Stream)owner.getData().getCardContainer() );
		
		getContentPane().add( mcdpanel, BorderLayout.CENTER );
		pack();
	}
	
	private void addnextcard() {
		// TODO: this needs to be updated to allow different card types.
		String s = mcdpanel.getEditedContent( );
		nextmetacard.setContents( s );
		String shortname = "\'" + s.substring( 0, s.indexOf( '\t' ) );
		if ( shortname.length() > 20 ) {
			shortname = shortname.substring( 0, 17 );
			shortname += "...\'";
		} else {
			shortname += "\'";
		}
		String msg = Strings.get(165) + shortname + 
			Strings.get(166) + owner.getCardContainerType() + Strings.get(167);
		cset.addMetaCard( nextmetacard );
		owner.onSaveCardSet( ); // TODO: Could be more elegant...
			ccontainer.add( cset.CreateCard( nextmetacard.getID() ) );
			owner.onSave( ); // TODO: Could be more elegant...
		owner.onSelectionChanged();
		
		ccdisplaypanel.redraw();
		messagelabel.setText( msg );
	}
	
	private void onAddContinue( ) {
		addnextcard();
		freshCard();
		mcdpanel.requestFocus();
	}

	private void onAddClose( ) {
		addnextcard();
		dispose( );
	}	
}
