package ankistream;

import java.io.File;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.util.Random;
import java.util.Vector;
import java.util.Iterator;
import java.util.Date;
import java.awt.Toolkit;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.Timer;
import javax.swing.ButtonGroup;

import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/**
 * The standard view module for the Anki Stream application. AnkiStreamFrame
 * represents a frame for editing and reviewing a Stream. <br>
 * <br>
 * Part of the Anki Stream project, a tool for long-term flash card review. Anki
 * Stream is distributed under the MIT/X11 free software license, a copy of
 * which should have come with this distribution.
 * 
 * @author Bishop Wilkins
 * @version 0.1
 */
public class AnkiStreamFrame extends JFrame {

	public static final String defaultDataFile = "AnkiStream.xml";

	private static final int PHASE_BROWSING = 0;

	private static final int PHASE_REVIEW = 1;

	private static final int PHASE_RESULT = 2;

	private int currentphase = PHASE_BROWSING;

	private AnkiStreamData data;

	private int selectedCompartment;

	private SummaryPanel summaryPanel;

	private CommonCardTablePanel cardTablePanel;

	private JPanel cardBrowserPanel;

	private CardContainerDisplayPanel displayPanel;

	private MetaCardDisplayPanel metacarddisplaypanel;

	private JPanel leftpanel;

	private JPanel reviewpanel;

	private ReviewControlPanel reviewcontrolpanel;

	private FindCardPanel findcardpanel;

	private JDialog waitDialog = null;
	private JLabel waitDialogLabel = null;

	// Card Timer
	private Timer cardtimer;

	private int secondspercard;

	private int secondsleft;

	private boolean cardcontainerchangesmade;

	private boolean cardsetchangesmade;

	// Files to which changes will be saved.
	// If null, user will be prompted for filenames.
	private File cardsfile = null;

	private File cardsetfile = null;

	// Handles for the components.
	private JMenu mFile;

	private JMenu mCardContainer;

	private JMenu mReview;

	private JMenu mHelp;

	// Handles for the controls.
	private JPanel upperpanel;

	private JPanel lowerpanel;

	// Browsing controls.
	private JTextField timelimitfield;

	private JCheckBox autosavecheckbox;

	private JLabel autosaveeveryendlabel;

	private JLabel secondspercardlabel;

	private JLabel notimelimitexplainlabel;

	private JTextField autosavefield;

	private JCheckBox autosaveafterreviewscheckbox;

	private JButton beginreviewbutton;

	private JButton beginlongtermreviewbutton;

	private JLabel messagelabel;

	// Review controls.
	private JLabel reviewlabel;

	private JButton flipbutton;

	private JButton rightbutton;

	private JButton wrongbutton;

	private JButton savetocardsetbutton;

	private JButton endreviewbutton;

	private JLabel timelabel;

	private JButton undobutton;

	private JPanel rpupperpanel;

	private JPanel rplowerpanel;

	// Browsing controls.
	private JLabel cardslabel;

	private JButton addcardsbutton;

	private JButton findcardbutton;

	private JButton importcardsbutton;

	private JButton exportcardsbutton;

	private JButton movecardsbutton;

	private JButton removecardsbutton;

	private JButton editcardbutton;

	private JButton reshufflebutton;

	// First startup tutorial comments.
	int tutorialphase = -1; // -1 = no tutorial, 0=add cards 1=now review

	boolean donotchangenamesyet;

	private boolean opennewfiledialogafterlayout = false;

	public AnkiStreamFrame() {

		donotchangenamesyet = true;

		cardBrowserPanel = null;

		// Set default fonts.
		// FontController.setDefaultFonts( FontController.getUnicodeFont() );

		// Load the localized strings.
		File basefilecheck = new File("strings");
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		// if ( !basefilecheck.exists() ) {
		try {
			BaseFileCreator.CreateStrings(baos);
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null,
					"Could not find or create base localization data file.",
					"Missing File Error", JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		// }
		Strings.load(baos);

		// Test for base data files.
		basefilecheck = new File("ankistreamdata.dtd");
		if (!basefilecheck.exists()) {
			try {
				BaseFileCreator.CreateDTD();
			} catch (IOException ioe) {
				JOptionPane.showMessageDialog(null, Strings.get(216), Strings
						.get(39), JOptionPane.ERROR_MESSAGE);
				System.exit(1);
			}
		}
		basefilecheck = new File("AnkiStream.xml");
		if (!basefilecheck.exists()) {
			try {
				BaseFileCreator.CreateXML();
			} catch (IOException ioe) {
				JOptionPane.showMessageDialog(null, Strings.get(217), Strings
						.get(39), JOptionPane.ERROR_MESSAGE);
				System.exit(1);
			}
		}

		// Load the data from file.
		data = AnkiStreamData.loadDataFromFile(defaultDataFile);
		if (data == null) {
			// Exit if base data files cannot be created.
			JOptionPane.showMessageDialog(null, Strings.get(218)
					+ defaultDataFile + Strings.get(219), Strings.get(39),
					JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}

		// Set language.
		Strings.setLanguage(data.getPreferences().getLanguage());

		// Run first startup sequence if necessary.
		if (data.getPreferences().getFirstStartup()) {
			tutorialphase = 0;
			/*
			 * AnkiStreamData.saveTagToFile( data.getPreferencesTag( ), new
			 * File( defaultDataFile ) ); final JDialog firsttimedialog = new
			 * JDialog(); firsttimedialog.setTitle( Strings.get(3) ); Container
			 * cpane = firsttimedialog.getContentPane(); cpane.setLayout(new
			 * BoxLayout(cpane, BoxLayout.Y_AXIS)); JPanel p = new JPanel(new
			 * FlowLayout(FlowLayout.CENTER,1,1)); p.add(new
			 * JLabel(Strings.get(220))); cpane.add(p); p = new JPanel(new
			 * FlowLayout(FlowLayout.CENTER,1,1)); p.add(new
			 * JLabel(Strings.get(221))); cpane.add(p); p = new JPanel(new
			 * FlowLayout(FlowLayout.CENTER,1,1)); p.add(new
			 * JLabel(Strings.get(222))); cpane.add(p); p = new JPanel( new
			 * FlowLayout() ); JButton yesbutton = new JButton( Strings.get(223) );
			 * yesbutton.addActionListener( new ActionListener() { public void
			 * actionPerformed( ActionEvent ae ) { opennewfiledialogafterlayout =
			 * true; firsttimedialog.dispose(); } } ); p.add( yesbutton );
			 * JButton nobutton = new JButton( Strings.get(224) );
			 * nobutton.addActionListener( new ActionListener() { public void
			 * actionPerformed( ActionEvent ae ) { opennewfiledialogafterlayout =
			 * false; firsttimedialog.dispose(); } } ); p.add( nobutton );
			 * cpane.add( p ); firsttimedialog.setModal( true );
			 * firsttimedialog.pack(); Dimension screenSize =
			 * Toolkit.getDefaultToolkit().getScreenSize();
			 * firsttimedialog.setLocation( (int) (screenSize.getWidth() / 2) -
			 * (firsttimedialog.getWidth() / 2), (int) (screenSize.getHeight() /
			 * 2) - (firsttimedialog.getHeight() / 2) );
			 * firsttimedialog.setVisible( true );
			 */
		}

		// Set up the waiting dialog.
		waitDialog = new JDialog(this, Strings.get(1), false);
		waitDialog.setLayout(new FlowLayout());
		waitDialogLabel = new JLabel(Strings.get(2));
		waitDialog.add(waitDialogLabel);
		Dimension ssize = Toolkit.getDefaultToolkit().getScreenSize();
		waitDialog.pack();
		waitDialog.setPreferredSize(new Dimension(200, 30));
		waitDialog.setLocation(new Point((ssize.width / 2) - 80,
				(ssize.height / 2) - 15));

		String cardsfilename = data.getPreferences().getDefaultStreamFile();
		cardsfile = new File(cardsfilename);
		if (data.getPreferences().getOpenDefaultStreamFileOnStartup()
				&& cardsfilename.compareToIgnoreCase(defaultDataFile) != 0) {
			// Load the real data file if it is different from defaultDataFile:
			loadNewCardContainer(cardsfile.getPath());
		}
		String cardsetfilename = data.getCardContainer().getCardSetFile();
		cardsetfile = new File(cardsetfilename);

		setTitle(Strings.get(3) + ": " + data.getCardContainer().getName());

		// Create the menu.
		JMenuBar mb = new JMenuBar();
		// File menu.
		JMenu m = new JMenu(Strings.get(8));
		m.setMnemonic(KeyEvent.VK_F);
		mFile = m;
		mb.add(m);
		// Stream menu.
		m = new JMenu(Strings.get(4));
		m.setMnemonic(KeyEvent.VK_S);
		mCardContainer = m;
		mb.add(m);
		// Review.
		m = new JMenu(Strings.get(6));
		m.setMnemonic(KeyEvent.VK_R);
		mReview = m;
		mb.add(m);
		// Help.
		m = new JMenu(Strings.get(14));
		m.setMnemonic(KeyEvent.VK_H);
		mHelp = m;
		mb.add(m);
		reloadMenuBar();

		getContentPane().setLayout(new BorderLayout());
		upperpanel = new JPanel(new BorderLayout());
		// Not sure if I really like the look of this border...
		// mb.setBorder( BorderFactory.createTitledBorder( "" ) );
		getContentPane().add(mb, BorderLayout.NORTH);

		// Construct controls.
		// Browsing controls.
		beginreviewbutton = new JButton(Strings.get(127));
		beginreviewbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onBeginReview();
			}
		});
		beginlongtermreviewbutton = new JButton("Begin Long Term Review");
		beginlongtermreviewbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onBeginLongTermReview();
			}
		});
		enableOrDisableLongTermReviewControls();

		// Review controls.
		flipbutton = new JButton(Strings.get(21));
		flipbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onFlip();
			}
		});
		flipbutton.addKeyListener(new AnkiBoxKeyListener());
		rightbutton = new JButton(Strings.get(22));
		rightbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onRight();
			}
		});
		rightbutton.addKeyListener(new AnkiBoxKeyListener());
		wrongbutton = new JButton(Strings.get(23));
		wrongbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onWrong();
			}
		});
		wrongbutton.addKeyListener(new AnkiBoxKeyListener());
		savetocardsetbutton = new JButton(Strings.get(25));
		savetocardsetbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onSaveToCardSet();
			}
		});
		endreviewbutton = new JButton(Strings.get(24));
		endreviewbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onEndReview();
			}
		});
		timelabel = new JLabel(Strings.get(26) + ": 00");
		undobutton = new JButton(Strings.get(27));
		undobutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onUndo();
			}
		});

		// Layout all panels.
		// Left panel.
		summaryPanel = new SummaryPanel(this);
		leftpanel = new JPanel(new BorderLayout());
		leftpanel.add(summaryPanel, BorderLayout.NORTH);

		onSelectionChanged(); // Add the initially empty card list panel.

		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.setBorder(BorderFactory.createTitledBorder(""));

		JPanel p2 = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		JLabel l = new JLabel(Strings.get(6) + ":");
		reviewlabel = l;
		l.setFont(l.getFont().deriveFont(Font.BOLD));
		p2.add(l);
		p.add(p2);

		p2 = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		p2.add(new JLabel("  "));
		secondspercardlabel = new JLabel(Strings.get(28) + ": ");
		p2.add(secondspercardlabel);
		secondspercard = data.getCardContainer().getSecondsPerCard();
		timelimitfield = new JTextField(Integer.toString(secondspercard), 3);
		// TODO: Should actually use a documentlistener or something for this.
		// I tried one before but it didn't seem to work...
		timelimitfield.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				onTimeLimitChanged();
			}
		});
		p2.add(timelimitfield);
		notimelimitexplainlabel = new JLabel(Strings.get(88));
		p2.add(notimelimitexplainlabel);
		p.add(p2);

		autosavecheckbox = new JCheckBox(Strings.get(86));
		autosavecheckbox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onUseAutoSaveEvery();
			}
		});
		autosavefield = new JTextField("100", 4);
		autosaveafterreviewscheckbox = new JCheckBox(Strings.get(29));
		autosaveafterreviewscheckbox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onAutoSaveAfterReviewsChecked();
			}
		});
		if (getData().getCardContainer() instanceof Stream) {
			autosavecheckbox
					.setSelected(((Stream) getData().getCardContainer())
							.getUseAutoSaveEvery());
			autosavefield.setText(String.valueOf(((Stream) getData()
					.getCardContainer()).getAutoSaveEvery()));
			autosaveafterreviewscheckbox.setSelected(((Stream) getData()
					.getCardContainer()).getAutoSaveAfterReview());
		}
		autosavefield.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				onAutoSaveEveryChanged();
			}
		});
		p2 = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		p2.add(new JLabel("  "));
		p2.add(autosavecheckbox);
		p2.add(autosavefield);
		autosaveeveryendlabel = new JLabel(Strings.get(87));
		p2.add(autosaveeveryendlabel);
		p.add(p2);
		p2 = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		p2.add(new JLabel("  "));
		p2.add(autosaveafterreviewscheckbox);
		p.add(p2);

		p2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
		p2.add(beginreviewbutton);
		if (data.getCardContainer() instanceof Box) {
			p2.add(beginlongtermreviewbutton);
		}
		p.add(p2);
		leftpanel.add(p, BorderLayout.SOUTH);
		getContentPane().add(leftpanel, BorderLayout.CENTER);
		// Review panel.
		reviewpanel = new JPanel(new BorderLayout());
		reviewcontrolpanel = new ReviewControlPanel();
		reviewpanel.add(reviewcontrolpanel, BorderLayout.SOUTH);
		// Right panel (box display).
		displayPanel = data.getCardContainer().getNewDisplay();
		displayPanel.notifyMeOnSelection(this);
		displayPanel.setCardContainer(data.getCardContainer());
		getContentPane().add((JPanel) displayPanel, BorderLayout.EAST);
		// Message label.
		lowerpanel = new JPanel(new FlowLayout());
		messagelabel = new JLabel(" ");
		lowerpanel.add(messagelabel);
		lowerpanel.setBorder(BorderFactory.createTitledBorder(""));
		getContentPane().add(lowerpanel, BorderLayout.SOUTH);
		messagelabel.setText(loadedmsg());

		if (opennewfiledialogafterlayout) {
			onNewStream();
		}

		// Center the frame and show.
		pack();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((int) (screenSize.getWidth() / 2) - (getWidth() / 2),
				(int) (screenSize.getHeight() / 2) - (getHeight() / 2));
		adjustDisplayPanelZoomToInitial();
		redrawDisplayPanel(true);
		displayPanel.redraw();
		setVisible(true);
		beginreviewbutton.requestFocus();

		setCardContainerChangesMade(false);
		setCardSetChangesMade(false);

		donotchangenamesyet = false; // TODO: this should not be necessary
		opennewfiledialogafterlayout = false; // TODO: neither should this

		// Start review if we loaded a box in the middle of one.
		if (data.getCardContainer().getCurrentCardID() != -1) {
			// TODO: onbeginreview for streams
			onBeginReview(((Box) data.getCardContainer())
					.getCurrentPartitionNumber(), false);
		}
	}

	// Reloads the menu bar items in case the review state or card container
	// type was changed and different options need to be displayed.
	private void reloadMenuBar() {

		// Clear everything first.
		mFile.removeAll();
		mCardContainer.removeAll();
		mReview.removeAll();
		mHelp.removeAll();

		mFile.setText(Strings.get(8));
		mCardContainer.setText(Strings.get(4));
		mReview.setText(Strings.get(6));
		mHelp.setText(Strings.get(14));

		CardContainer cc = data.getCardContainer();

		// File Menu
		String s;
		if (cc instanceof Stream) {
			s = Strings.get(9);
		} else {
			s = Strings.get(10);
		}
		JMenuItem mi = new JMenuItem(s);
		JMenu m = mFile;
		mi.setMnemonic(KeyEvent.VK_N);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onNewStream();
			}
		});
		m.add(mi);
		mi = new JMenuItem(Strings.get(11));
		mi.setMnemonic(KeyEvent.VK_O);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onOpen();
			}
		});
		m.add(mi);
		mi = new JMenuItem(Strings.get(12));
		mi.setMnemonic(KeyEvent.VK_S);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onSave();
			}
		});
		m.add(mi);
		mi = new JMenuItem(Strings.get(13));
		mi.setMnemonic(KeyEvent.VK_A);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onSaveAs();
			}
		});
		m.add(mi);

		// Stream/Box Menu
		m = mCardContainer;
		m.setName(getCardContainerType());
		if (cc instanceof Box) {
			mi = new JMenuItem(Strings.get(30));
			mi.setMnemonic(KeyEvent.VK_P);
			mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					String s = "Please specify card capacities for your box partitions."
							+ "\n(Numbers separated by commas.)"
							+ "\nYou can specify as many or as few partitions as desired."
							+ "\nAny cards orphaned by the removal of their partition will be"
							+ "\nmoved to the partition closest to the back of the box.";
					PartitionSizesDialog psd = new PartitionSizesDialog(
							AnkiStreamFrame.this, s);
					if (!psd.usercancelled) {
						data.getBox().rePartition(psd.sizes);
						redrawDisplayPanel();
						messagelabel.setText("Resized box partitions.");
					}
				}
			});
			m.add(mi);
		}
		mi = new JMenuItem(Strings.get(15));
		mi.setMnemonic(KeyEvent.VK_C);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onAddCards();
			}
		});
		m.add(mi);
		mi = new JMenuItem(Strings.get(16));
		mi.setMnemonic(KeyEvent.VK_F);
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onFindCards();
			}
		});
		m.add(mi);
		if (cc instanceof Stream) {
			m.addSeparator();
			mi = new JMenuItem(Strings.get(17));
			mi.setMnemonic(KeyEvent.VK_F);
			mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onImportCards();
				}
			});
			m.add(mi);
			mi = new JMenuItem(Strings.get(18));
			mi.setMnemonic(KeyEvent.VK_F);
			mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onExportCards();
				}
			});
			m.add(mi);
		}

		// Review Menu
		m = mReview;
		if (cc.getCurrentCardID() == -1) {
			mi = new JMenuItem(Strings.get(19));
			mi.setMnemonic(KeyEvent.VK_B);
			mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onBeginReview();
				}
			});
			m.add(mi);
			if (cc instanceof Box) {
				mi = new JMenuItem(Strings.get(20));
				mi.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						onBeginLongTermReview();
					}
				});
				m.add(mi);
			}
		} else {
			if (currentphase == PHASE_REVIEW) {
				mi = new JMenuItem(Strings.get(21));
				mi.setMnemonic(KeyEvent.VK_F);
				mi.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						onFlip();
					}
				});
				m.add(mi);
			} else { // PHASE_RESULT
				mi = new JMenuItem(Strings.get(22));
				mi.setMnemonic(KeyEvent.VK_R);
				mi.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						onRight();
					}
				});
				m.add(mi);
				mi = new JMenuItem(Strings.get(23));
				mi.setMnemonic(KeyEvent.VK_W);
				mi.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						onWrong();
					}
				});
				m.add(mi);
			}
			mi = new JMenuItem(Strings.get(27));
			mi.setMnemonic(KeyEvent.VK_W);
			mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onUndo();
				}
			});
			m.add(mi);
			m.addSeparator();
			mi = new JMenuItem(Strings.get(24));
			if (cc instanceof Box) {
				s = "Abort the current review session:\n"
						+ "Move any waiting cards into their target\n"
						+ "partition.  Partitions will not be reshuffled.";
				mi.setToolTipText(s);
			}
			mi.setMnemonic(KeyEvent.VK_B);
			mi.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onEndReview();
				}
			});
			m.add(mi);
		}

		// Help.
		m = mHelp;
		m.setMnemonic(KeyEvent.VK_H);
		mi = new JMenuItem(Strings.get(31));
		m.add(mi);
		String[] langs = Strings.getLanguages();
		ButtonGroup bg = new ButtonGroup();
		for (int c = 0; c < langs.length; c++) {
			LanguageButton lb = new LanguageButton(langs[c], c);
			bg.add(lb);
			m.add(lb);
			if (Strings.getLanguage() == c) {
				lb.setSelected(true);
			}
		}
		m.addSeparator();
		mi = new JMenuItem(Strings.get(32));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				new AboutDialog();
			}
		});
		m.add(mi);
	}

	class LanguageButton extends JRadioButtonMenuItem implements ActionListener {

		int languageNumber = 0;

		public LanguageButton(String s, int newLanguageNumber) {
			super(s);
			languageNumber = newLanguageNumber;
			addActionListener(this);
		}

		public void actionPerformed(ActionEvent ae) {
			onSetLanguage(languageNumber);
		}
	}

	private void onSetLanguage(int languageNumber) {

		Strings.setLanguage(languageNumber);

		// Remember the new language.
		// TODO: encapsulate these preference changes in a subroutine.
		data.getPreferences().setLanguage(languageNumber);
		AnkiStreamData.saveTagToFile(data.getPreferencesTag(), new File(
				defaultDataFile));

		// Redraw all the strings.
		setTitle(Strings.get(3) + ": " + data.getCardContainer().getName());
		reloadMenuBar();
		summaryPanel.updateSummary();
		onSelectionChanged();
		
		waitDialog.setTitle(Strings.get(1));
		waitDialogLabel.setText(Strings.get(2));

		displayPanel.redraw();

		// Review controls.
		reviewlabel.setText(Strings.get(6) + ":");
		autosavecheckbox.setText(Strings.get(86));
		autosaveeveryendlabel.setText(Strings.get(87));
		autosaveafterreviewscheckbox.setText(Strings.get(29));
		secondspercardlabel.setText(Strings.get(28) + ": ");
		notimelimitexplainlabel.setText(Strings.get(88));
		beginreviewbutton.setText(Strings.get(19));
		beginlongtermreviewbutton.setText(Strings.get(20));
		messagelabel.setText(Strings.get(201));
		flipbutton.setText(Strings.get(21));
		rightbutton.setText(Strings.get(22));
		wrongbutton.setText(Strings.get(23));
		savetocardsetbutton.setText(Strings.get(25));
		endreviewbutton.setText(Strings.get(24));
		timelabel.setText(Strings.get(26) + ": " + secondsleft);
		undobutton.setText(Strings.get(27));
		// Box browsing controls.
		cardslabel.setText(Strings.get(5) + ":");
		if (addcardsbutton != null) {
			addcardsbutton.setText(Strings.get(59));
		}
		if (findcardbutton != null) {
			findcardbutton.setText(Strings.get(60));
		}
		if (importcardsbutton != null) {
			importcardsbutton.setText(Strings.get(61));
		}
		if (exportcardsbutton != null) {
			exportcardsbutton.setText(Strings.get(62));
		}
		if (movecardsbutton != null) {
			movecardsbutton.setText(Strings.get(63));
		}
		if (removecardsbutton != null) {
			removecardsbutton.setText(Strings.get(64));
		}
		if (editcardbutton != null) {
			editcardbutton.setText(Strings.get(65));
		}
		if (reshufflebutton != null) {
			reshufflebutton.setText(Strings.get(66));
		}

		repaint();
	}

	private void startReviewTimer() {
		try {
			secondspercard = Integer.parseInt(timelimitfield.getText());
		} catch (NumberFormatException nfe) {
			secondspercard = 0;
		}
		if (secondspercard == 0) { // no timer desired.
			timelabel.setText("");
			return;
		}
		if (cardtimer != null) {
			cardtimer.stop();
		}
		secondsleft = secondspercard;
		ActionListener timerListener = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				secondsleft--;
				timelabel.setText(Strings.get(26) + ": " + secondsleft);
				if (secondsleft <= 0) {
					onFlip();
					wrongbutton.requestFocus();
					rightbutton.setEnabled(false);
				}
			}
		};
		timelabel.setText(Strings.get(26) + ": " + secondsleft);
		cardtimer = new Timer(1000, timerListener);
		cardtimer.start();
	}

	private void stopReviewTimer() {
		if (secondspercard == 0)
			return; // no timer desired.
		if (cardtimer != null)
			cardtimer.stop();
	}

	public void setMessage(String newvalue) {
		messagelabel.setText(newvalue);
	}

	public void setWaitDialogVisible(boolean value) {
		setWaitDialogVisible(value, "");
	}

	public void setWaitDialogVisible(boolean value, String message) {
		if (waitDialog != null) {
			if (value) {
				waitDialogLabel.setText(Strings.get(2));
				waitDialog.setTitle(message);
				waitDialog.pack();
				if (waitDialog.getWidth() < 200) {
					waitDialog.setSize(new Dimension(200, waitDialog
							.getHeight()));
				}
			}
			waitDialog.setVisible(value);
			waitDialog.toFront();
		}
	}

	/**
	 * Call to set the cardcontainerchangesmade value.
	 */
	public void setCardContainerChangesMade(boolean newvalue) {
		cardcontainerchangesmade = newvalue;
	}

	/**
	 * Call to set the cardcontainerchangesmade value.
	 */
	public void setCardSetChangesMade(boolean newvalue) {
		cardsetchangesmade = newvalue;
	}

	public void redrawDisplayPanel() {
		redrawDisplayPanel(false);
	}

	/**
	 * Redraws the display panel.
	 * 
	 * @param autoAdjustRange
	 *            should the display panel's range be auto- adjusted to show the
	 *            current and most recently reviewed cards?
	 */
	public void redrawDisplayPanel(boolean autoAdjustRange) {
		if ((displayPanel instanceof StreamDisplayPanel) && autoAdjustRange) {
			((StreamDisplayPanel) displayPanel).showCurrentAndLastReviewed();
			displayPanel.redraw();
		} else {
			displayPanel.redraw();
		}
	}

	/**
	 * Called when the user changes the time limit value field.
	 */
	public void onTimeLimitChanged() {
		int newtimelimit = 0;
		try {
			newtimelimit = Integer.parseInt(timelimitfield.getText());
		} catch (NumberFormatException nfe) {
			timelimitfield.setText(Integer.toString(secondspercard));
			return;
		}
		// Set new time limit, mark card container for changes to be saved.
		if (secondspercard != newtimelimit) {
			secondspercard = newtimelimit;
			data.getCardContainer().setSecondsPerCard(secondspercard);
			setCardContainerChangesMade(true);
		}
	}

	private void onUseAutoSaveEvery() {
		Stream s = (Stream) getData().getCardContainer();
		boolean oldval = s.getUseAutoSaveEvery();
		s.setUseAutoSaveEvery(autosavecheckbox.isSelected());
		if (s.getUseAutoSaveEvery() != oldval) {
			setCardContainerChangesMade(true);
		}
	}

	private void onAutoSaveEveryChanged() {
		Stream s = (Stream) getData().getCardContainer();
		int oldval = s.getAutoSaveEvery();
		int newval = -1;
		try {
			newval = Integer.parseInt(autosavefield.getText());
		} catch (Exception e) {
			newval = -1;
		}
		if (newval < 1 || newval > 1000000) {
			autosavefield.setText(String.valueOf(oldval));
			return;
		}
		s.setAutoSaveEvery(newval);
		if (s.getAutoSaveEvery() != oldval) {
			setMessage(Strings.get(33) + newval + Strings.get(34));
			setCardContainerChangesMade(true);
		}
	}

	private void onAutoSaveAfterReviewsChecked() {
		Stream s = (Stream) getData().getCardContainer();
		boolean oldval = s.getAutoSaveAfterReview();
		s.setAutoSaveAfterReview(autosaveafterreviewscheckbox.isSelected());
		if (s.getAutoSaveAfterReview() != oldval) {
			setCardContainerChangesMade(true);
		}
	}

	/**
	 * Called when the user changes the card container name.
	 */
	public void onCCNameChanged() {
		if (donotchangenamesyet)
			return;
		String newname = summaryPanel.namefield.getText();
		if (newname.trim().length() < 1) {
			// Invalid name specified.
			summaryPanel.namefield.setText(data.getCardContainer().getName());
		}
		data.getCardContainer().setName(newname);
		setTitle(Strings.get(3) + ": " + data.getCardContainer().getName());
		messagelabel.setText(getCardContainerType() + Strings.get(35) + newname
				+ Strings.get(36));
	}

	/**
	 * Called when the user clicks "New" on the menu bar.
	 */
	public boolean onNewStream() {

		if (!confirmClose())
			return false;

		JFileChooser nsd = new JFileChooser();
		nsd.setSelectedFile(new File("NewStream.xml"));
		nsd.setApproveButtonText(Strings.get(84));
		nsd.setDialogTitle(Strings.get(227));
		nsd.setName(Strings.get(227));
		int rval = nsd.showSaveDialog(this);
		if (rval != JFileChooser.APPROVE_OPTION) {
			return false;
		}

		// Set file names.
		cardsfile = nsd.getSelectedFile();
		cardsetfile = cardsfile;

		// Create/load card set.
		// Attempt to load a pre-existing card set if the specified card set
		// file already existed.
		AnkiStreamData loadeddata = AnkiStreamData.loadDataFromFile(cardsetfile
				.getAbsolutePath());
		if (loadeddata == null) {
			data.setCardSet(new CardSet());
		} else {
			data.setCardSet(loadeddata.getCardSet());
		}
		AnkiStreamData.saveTagToFile(data.getCardSetTag(), cardsetfile);

		// Create stream.
		Stream newstream = new Stream(data.getCardSet().getName());
		newstream.setCardSetFile(cardsetfile.getPath());
		data.setStream(newstream);
		displayPanel.setCardContainer(newstream);
		summaryPanel.updateSummary();
		repaint();
		setTitle(Strings.get(3) + ": " + data.getStream().getName());

		// Save stream to file.
		onSave();

		// Redraw the new stream.
		if (displayPanel != null && !opennewfiledialogafterlayout) {
			displayPanel.setCardContainer(data.getCardContainer());
			redrawDisplayPanel();
			summaryPanel.updateSummary();
			repaint();
		}

		if (messagelabel != null) {
			String msg = Strings.get(225) + cardsfile.getName()
					+ Strings.get(226);
			messagelabel.setText(msg);
		}

		return true;
	}

	/**
	 * Called when the user requests a new anki box. Returns true if the new box
	 * was created, false otherwise.
	 */
	public boolean onNewBox() {
		if (!confirmClose())
			return false;

		NewBoxDialog nbd = new NewBoxDialog(this);
		if (nbd.usercancelled)
			return false;

		String s = "Please specify card capacities for your box partitions."
				+ "\n(Numbers separated by commas.)"
				+ "\nYou can specify as many or as few partitions as desired.";
		PartitionSizesDialog psd = new PartitionSizesDialog(this, s);
		if (psd.usercancelled)
			return false;

		// Set file names.
		File oldstreamfile = cardsfile;
		File oldcardsetfile = cardsetfile;
		cardsfile = new File(nbd.boxfilename);
		cardsetfile = new File(nbd.cardsetfilename);

		// Create/load card set.
		// Attempt to load a pre-existing card set if the specified card set
		// file already existed.
		AnkiStreamData loadeddata = AnkiStreamData.loadDataFromFile(cardsetfile
				.getAbsolutePath());
		if (loadeddata == null) {
			data.setCardSet(new CardSet());
		} else {
			data.setCardSet(loadeddata.getCardSet());
		}
		AnkiStreamData.saveTagToFile(data.getCardSetTag(), cardsetfile);

		// Create box.
		Box newbox = new Box(data.getCardSet().getName(), psd.sizes);
		newbox.setCardSetFile(cardsetfile.getPath());
		// In case there was a stream loaded that would override this box:
		data.setStream(null);
		data.setBox(newbox);
		displayPanel.setCardContainer(newbox);
		summaryPanel.updateSummary();
		repaint();
		setTitle(Strings.get(3) + ": " + data.getBox().getName());

		// Save box to file.
		onSaveBox();

		// Redraw the new box.
		if (displayPanel != null && !opennewfiledialogafterlayout) {
			displayPanel.setCardContainer(data.getCardContainer());
			redrawDisplayPanel();
			summaryPanel.updateSummary();
			repaint();
		}

		if (messagelabel != null) {
			String msg = "Created new box.  Using box file \'"
					+ cardsfile.getName() + "\', and card set file \'"
					+ cardsetfile.getName() + "\'.";
			messagelabel.setText(msg);
		}

		return true;
	}

	/**
	 * Called when the user clicks "Open" on the menu bar.
	 */
	public void onOpen() {
		if (!confirmClose())
			return;

		JFileChooser fc = new JFileChooser();
		fc.setCurrentDirectory(new File("."));
		int result = fc.showOpenDialog(this);
		if (result == JFileChooser.APPROVE_OPTION) {

			// Stop the review if there is one. Do not save (again).
			if (data.getCardContainer().getCurrentCardID() != -1) {
				onEndReview();
			}

			// Load the new card container.
			String file = fc.getSelectedFile().getPath();
			loadNewCardContainer(file);
			setCardContainerChangesMade(false);
			setCardSetChangesMade(false);
			setTitle(Strings.get(3) + ": " + data.getCardContainer().getName());
			timelimitfield.setText(String.valueOf(data.getCardContainer()
					.getSecondsPerCard()));
			if (data.getCardContainer().getCurrentCardID() == -1) {
				// No review in progress.
				beginreviewbutton.requestFocus();
			} else {
				onBeginReview();
			}
		}
	}

	// returns true if the new card container was loaded sucessfully.
	private boolean loadNewCardContainer(String file) {

		// Put up a "please wait" dialog if the file is > 1 mb.
		File f = new File(file);
		if (f == null) {
			return false;
		}
		if (f.length() > 1000000) {
			waitDialog.setTitle(Strings.get(1));
			waitDialog.setVisible(true);
		}

		// Load file.
		AnkiStreamData newdata = AnkiStreamData.loadDataFromFile(file);

		if (newdata == null) {
			JOptionPane.showMessageDialog(null, Strings.get(37) + file
					+ Strings.get(38), Strings.get(39),
					JOptionPane.ERROR_MESSAGE);
			if (waitDialog != null) {
				waitDialog.setVisible(false);
			}
			return false;
		}
		CardContainer newcc = newdata.getCardContainer();
		if (newcc == null) {
			JOptionPane.showMessageDialog(null, Strings.get(40) + file
					+ Strings.get(41), Strings.get(39),
					JOptionPane.ERROR_MESSAGE);
			if (waitDialog != null) {
				waitDialog.setVisible(false);
			}
			return false;
		}
		data.getPreferences().setDefaultStreamFile(file);
		data.setCardContainer(newcc);
		cardsfile = new File(file);

		// Load the corresponding card set.
		String cardsetfilename = data.getCardContainer().getCardSetFile();
		newdata = AnkiStreamData.loadDataFromFile(cardsetfilename);
		if (newdata == null) {
			JOptionPane.showMessageDialog(null, Strings.get(42)
					+ cardsetfilename + Strings.get(43), Strings.get(39),
					JOptionPane.ERROR_MESSAGE);
			if (waitDialog != null) {
				waitDialog.setVisible(false);
			}
			return false;
		}
		CardSet newcardset = newdata.getCardSet();
		if (newcardset == null) {
			JOptionPane.showMessageDialog(null, Strings.get(44), Strings
					.get(39), JOptionPane.ERROR_MESSAGE);
			if (waitDialog != null) {
				waitDialog.setVisible(false);
			}
			return false;
		}
		data.setCardSet(newcardset);

		cardsetfile = new File(cardsetfilename);

		// Save the new default stream preference
		data.getPreferences().setDefaultStreamFile(cardsfile.getPath());
		AnkiStreamData.saveTagToFile(data.getPreferencesTag(), new File(
				defaultDataFile));

		// Redraw the new container.
		if (displayPanel != null) {

			// Convert to the right type of card container display first.
			getContentPane().remove((JPanel) displayPanel);
			displayPanel = data.getCardContainer().getNewDisplay();
			displayPanel.notifyMeOnSelection(this);
			displayPanel.setCardContainer(data.getCardContainer());
			adjustDisplayPanelZoomToInitial();
			getContentPane().add((JPanel) displayPanel, BorderLayout.EAST);
			pack();
			redrawDisplayPanel(true);

			// Try to rip out and replace the summary panel.
			leftpanel.remove(summaryPanel);
			summaryPanel = new SummaryPanel(this);
			leftpanel.add(summaryPanel, BorderLayout.NORTH);
			summaryPanel.updateSummary();

			// Update review options too.
			if (data.getCardContainer() instanceof Stream) {
				Stream stream = (Stream) getData().getCardContainer();
				autosavecheckbox.setSelected(stream.getUseAutoSaveEvery());
				autosavefield
						.setText(String.valueOf(stream.getAutoSaveEvery()));
				autosaveafterreviewscheckbox.setSelected(stream
						.getAutoSaveAfterReview());
			}

			repaint();
		}

		// Display box loaded message:
		if (messagelabel != null) {
			messagelabel.setText(loadedmsg());
		}

		if (waitDialog != null) {
			waitDialog.setVisible(false);
		}

		return true;
	}

	private void adjustDisplayPanelZoomToInitial() {
		if (displayPanel instanceof StreamDisplayPanel) {
			String s = ((Stream) data.getCardContainer()).getInitialZoom();
			if (s.equalsIgnoreCase("auto")) {
				((StreamDisplayPanel) displayPanel).setAutoZoom(true);
			} else {
				((StreamDisplayPanel) displayPanel).setAutoZoom(false);
				int initialval = 50;
				try {
					initialval = Integer.parseInt(s);
					initialval = Math.min(initialval, 10000);
					initialval = Math.max(initialval, 1);
				} catch (Exception e) {
				}
				((StreamDisplayPanel) displayPanel).setZoomSlider(initialval);
			}
		}
	}

	private String loadedmsg() {
		return Strings.get(45) + getCardContainerTypeLowerCase()
				+ Strings.get(46) + cardsfile.getName() + Strings.get(47);
	}

	/**
	 * Gets the AnkiBoxData containing for this Anki Box frame.
	 * 
	 * @return the AnkiBoxData object for this Anki Box frame.
	 */
	public AnkiStreamData getData() {
		return data;
	}

	/**
	 * Prompts the user to save/not save/cancel if changes have been made.
	 * 
	 * @return true if the user wants to go on, false if the user cancelled.
	 */
	public boolean confirmClose() {
		if (cardcontainerchangesmade || cardsetchangesmade) {
			String msg = Strings.get(48);
			int choice = JOptionPane.showConfirmDialog(this, msg, Strings
					.get(49), JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.WARNING_MESSAGE);
			switch (choice) {
			case JOptionPane.YES_OPTION:
				if (cardcontainerchangesmade) {
					onSave();
				}
				if (cardsetchangesmade) {
					onSaveCardSet();
				}
				return true;
			case JOptionPane.NO_OPTION:
				return true;
			default:
				return false;
			}
		}
		return true;
	}

	/**
	 * Called when the user clicks "Save" on the menu bar.
	 */
	public boolean onSave() {
		// Get a new filename if the current cardcontainer is unnamed.
		if (cardsfile == null) {
			if (!onSaveAs()) {
				return false;
			}
		}

		// Show wait dialog if this is a big cardcontainer.
		if (data.getCardContainer().getCardCount() > 999) {
			waitDialog.setTitle(Strings.get(243));
			waitDialog.setVisible(true);
		}

		// Set the default box in preferences to the new filename.
		data.getPreferences().setDefaultStreamFile(cardsfile.getPath());
		AnkiStreamData.saveTagToFile(data.getPreferencesTag(), new File(
				defaultDataFile));

		// Save the cardcontainer.
		File f = new File(cardsfile.getPath());
		data.setCardContainer(this.getData().getCardContainer());
		if (!AnkiStreamData.saveTagToFile(data.getCardContainer().toXML(), f)) {
			JOptionPane.showMessageDialog(this, Strings.get(50), Strings
					.get(39), JOptionPane.ERROR_MESSAGE);
			if (waitDialog != null) {
				waitDialog.setVisible(false);
			}
			return false;
		}

		// No changes made since last save.
		setCardContainerChangesMade(false);

		// Print box saved message:
		String msg = getCardContainerSavedMessage();
		messagelabel.setText(msg);

		if (waitDialog != null) {
			waitDialog.setVisible(false);
			if (data.getCardContainer().getCurrentCardID() != -1) {
				flipbutton.requestFocus();
			}
		}

		return true;
	}

	/**
	 * Called when the user requests the anki box to be saved.
	 * 
	 * @return true if the box was saved, false otherwise.
	 */
	public boolean onSaveBox() {

		// Get a new box filename if the current box is unnamed.
		if (cardsfile == null) {
			if (!onSaveBoxAs()) {
				return false;
			}
		}

		// Set the default box in preferences to the new filename.
		data.getPreferences().setDefaultStreamFile(cardsfile.getPath());
		AnkiStreamData.saveTagToFile(data.getPreferencesTag(), new File(
				defaultDataFile));

		// Save the box.
		File f = new File(cardsfile.getPath());
		if (!AnkiStreamData.saveTagToFile(data.getBoxTag(), f)) {
			JOptionPane.showMessageDialog(this, Strings.get(50), Strings
					.get(39), JOptionPane.ERROR_MESSAGE);
			return false;
		}

		// No changes made since last save.
		setCardContainerChangesMade(false);

		// Print box saved message:
		String msg = getCardContainerSavedMessage();
		messagelabel.setText(msg);

		return true;
	}

	private String getCardContainerSavedMessage() {
		return Strings.get(51) + cardsfile.getName() + Strings.get(52);
	}

	/**
	 * Called when the user clicks "Save As" on the menu bar.
	 */
	public boolean onSaveAs() {

		JFileChooser jfc = new JFileChooser();

		int result = jfc.showSaveDialog(this);

		if (result == JFileChooser.APPROVE_OPTION) {
			cardsfile = jfc.getSelectedFile();
			onSave();
			return true;
		}

		return false;
	}

	/**
	 * Called when the user requests the anki box to be saved as something.
	 * 
	 * @return true if the user wishes to proceed with the save, false
	 *         otherwise.
	 */
	public boolean onSaveBoxAs() {

		JFileChooser jfc = new JFileChooser();

		int result = jfc.showSaveDialog(this);

		if (result == JFileChooser.APPROVE_OPTION) {
			cardsfile = jfc.getSelectedFile();
			onSaveBox();
			return true;
		}

		return false;
	}

	/**
	 * Called when the user requests to search for a card.
	 */
	public void onFindCards() {
		// Open the find card panel if there is no review in progress.
		if (data.getCardContainer().getCurrentCardID() == -1) {
			findcardpanel = new FindCardPanel(this);
		}
	}

	/**
	 * Called when the user requests to add cards to the card set.
	 */
	public void onAddCards() {
		// Open the find card panel if there is no review in progress.
		if (data.getCardContainer().getCurrentCardID() == -1) {
			// Deselect any selected compartment if one is selected.
			if (displayPanel.getSelectedCardID() != -1) {
				displayPanel.onClick(0, 0);
			}
			new AddCardsDialog(this, data.getCardContainer(),
					data.getCardSet(), displayPanel);
			if ((tutorialphase != -1)
					&& (data.getCardContainer().getCardCount() > 0)) {
				// User's first time use.
				// Display more help after card are added.
				final JDialog explainreviewdialog = new JDialog();
				explainreviewdialog.setTitle(Strings.get(3));
				Container cpane = explainreviewdialog.getContentPane();
				cpane.setLayout(new BoxLayout(cpane, BoxLayout.Y_AXIS));
				for (int c = 231; c < 243; c++) {
					JPanel p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1,
							1));
					p.add(new JLabel(Strings.get(c)));
					cpane.add(p);
				}
				JPanel p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
				JButton okbutton = new JButton(Strings.get(84));
				okbutton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						explainreviewdialog.dispose();
					}
				});
				p.add(okbutton);
				cpane.add(p);
				explainreviewdialog.setModal(true);
				explainreviewdialog.pack();
				Dimension screenSize = Toolkit.getDefaultToolkit()
						.getScreenSize();
				explainreviewdialog.setLocation(
						(int) (screenSize.getWidth() / 2)
								- (explainreviewdialog.getWidth() / 2),
						(int) (screenSize.getHeight() / 2)
								- (explainreviewdialog.getHeight() / 2));
				explainreviewdialog.setVisible(true);
				data.getPreferences().setFirstStartup(false);
				AnkiStreamData.saveTagToFile(data.getPreferencesTag(),
						new File(defaultDataFile));
				tutorialphase = -1; // end of tutorial.
			}
		}
	}

	/**
	 * Called when the user requests to import cards to the card set.
	 */
	public void onImportCards() {
		// Open the import cards panel if there is no review in progress.
		if (data.getCardContainer().getCurrentCardID() == -1) {
			// Deselect any selected compartment if one is selected.
			if (displayPanel.getSelectedCardID() != -1) {
				displayPanel.onClick(0, 0);
			}
			new ImportCardsDialog(this);
		}
	}

	/**
	 * Called when the user requests to export cards to the card set.
	 */
	public void onExportCards() {
		new ExportCardsDialog(this);
	}

	/**
	 * Called when the user requests the card set to be saved.
	 */
	public boolean onSaveCardSet() {

		// Get a new card set filename if the current card set is unnamed.
		// I can't think of any time this should ever happen.
		if (cardsetfile == null) {
			if (!onSaveBoxAs()) {
				return false;
			}
		}

		// Set the current box to use the new card set filename.
		data.getCardContainer().setCardSetFile(cardsetfile.getAbsolutePath());

		// Save the card set.
		File f = new File(cardsetfile.getPath());
		if (!AnkiStreamData.saveTagToFile(data.getCardSetTag(), f)) {
			JOptionPane.showMessageDialog(this, Strings.get(50), Strings
					.get(39), JOptionPane.ERROR_MESSAGE);
			return false;
		}

		// No carset changes made since last save.
		setCardSetChangesMade(false);

		// Print card saved message:
		String msg = Strings.get(53) + cardsetfile.getName() + Strings.get(54);
		messagelabel.setText(msg);

		return true;
	}

	// Removes anything from the center of the left panel
	private void clearLeftPanel() {
		if (cardBrowserPanel != null) {
			leftpanel.remove(cardBrowserPanel);
			cardBrowserPanel = null;
		}
	}

	/**
	 * Called when the user selects or deselects a compartment while browsing
	 * the box.
	 */
	public void onSelectionChanged() {

		int sid;

		if (displayPanel == null) {

			sid = -1;

		} else {
			// TODO: Big fix for small issue: multiple instances of the same
			// card/metacard id will have weird behavior... distinguish between
			// card and metacard id attributes on each card?
			sid = displayPanel.getSelectedCardID();
		}

		// Remove the old browsing panel if necessary.
		clearLeftPanel();

		cardBrowserPanel = new JPanel();
		cardBrowserPanel.setLayout(new BoxLayout(cardBrowserPanel,
				BoxLayout.Y_AXIS));
		JPanel p = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		JLabel l = new JLabel(Strings.get(5) + ":");
		cardslabel = l;
		l.setFont(l.getFont().deriveFont(Font.BOLD));
		p.add(l);
		cardBrowserPanel.add(p);

		// Place card adding tutorial message if there are no cards.
		if (data.getCardContainer().getCardCount() < 1) {
			p = new JPanel();
			p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
			p.setBorder(BorderFactory.createLineBorder(Color.BLACK));
			p.setBackground(Color.LIGHT_GRAY);
			JPanel p2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
			p2.setBackground(Color.LIGHT_GRAY);
			p2.add(new JLabel(Strings.get(55)));
			p.add(p2);
			p2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
			p2.setBackground(Color.LIGHT_GRAY);
			p2.add(new JLabel(Strings.get(56)));
			p.add(p2);
			p2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
			p2.setBackground(Color.LIGHT_GRAY);
			p2.add(new JLabel(Strings.get(57)));
			p.add(p2);
			p2 = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
			p2.setBackground(Color.LIGHT_GRAY);
			p2.add(new JLabel(Strings.get(58)));
			p.add(p2);
			cardBrowserPanel.add(p);
		}

		// Place add/find/import/export buttons.
		p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
		addcardsbutton = new JButton(Strings.get(59));
		addcardsbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onAddCards();
			}
		});
		p.add(addcardsbutton);
		p.add(new JLabel("  "));
		findcardbutton = new JButton(Strings.get(60));
		findcardbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onFindCards();
			}
		});
		p.add(findcardbutton);
		cardBrowserPanel.add(p);

		p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
		importcardsbutton = new JButton(Strings.get(61));
		importcardsbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onImportCards();
			}
		});
		p.add(importcardsbutton);
		p.add(new JLabel("  "));
		exportcardsbutton = new JButton(Strings.get(62));
		exportcardsbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onExportCards();
			}
		});
		p.add(exportcardsbutton);
		cardBrowserPanel.add(p);

		if (sid != -1) {
			// Place browsing table and edit/move/remove buttons.
			Vector clist = displayPanel.getCardRange();
			cardTablePanel = data.getCardContainer().getNewTablePanel(clist,
					data.getCardSet(), this);
			cardBrowserPanel.add((JPanel) cardTablePanel);

			p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			editcardbutton = new JButton(Strings.get(65));
			editcardbutton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onEditCard();
				}
			});
			p.add(editcardbutton);
			p.add(new JLabel(" "));
			movecardsbutton = new JButton(Strings.get(63));
			movecardsbutton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					JOptionPane.showMessageDialog(null, Strings.get(207),
							Strings.get(63), JOptionPane.INFORMATION_MESSAGE);
					((StreamDisplayPanel) displayPanel).initiateMove();
				}
			});
			p.add(movecardsbutton);
			p.add(new JLabel(" "));
			removecardsbutton = new JButton(Strings.get(64));
			removecardsbutton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onRemoveCards();
				}
			});
			p.add(removecardsbutton);
			cardBrowserPanel.add(p);

			if (cardTablePanel instanceof StreamCardTablePanel) {
				((StreamCardTablePanel) cardTablePanel).selectAll();
			}
		}

		cardBrowserPanel.setBorder(BorderFactory.createTitledBorder(""));
		//cardBrowserPanel.setPreferredSize(new Dimension(320, 120));
		//cardBrowserPanel.setPreferredSize(new Dimension(320, 80));
		leftpanel.add(cardBrowserPanel, BorderLayout.CENTER);
		pack(); // Required for some reason to get the new table to show
	}

	/**
	 * Called when the cards selected on a card table panel change.
	 */
	public void onCardTableSelectionChanged() {
		int[] cards = cardTablePanel.getSelectedCardIDs();
		if (cards.length > 0) {
			movecardsbutton.setEnabled(true);
			removecardsbutton.setEnabled(true);
			editcardbutton.setEnabled(true);
		} else {
			movecardsbutton.setEnabled(false);
			removecardsbutton.setEnabled(false);
			editcardbutton.setEnabled(false);
		}
	}

	/** Moves the cards selected in the cardTablePanel to the specified time. */
	public void onMoveSelectedCardsTo(long newTime, Vector selectedCards) {
		Stream stream = ((Stream) data.getCardContainer());
		int[] cardids = cardTablePanel.getSelectedCardIDs();
		selectedCards.removeAllElements();
		for (int i = 0; i < cardids.length; i++) {
			int id = cardids[i];
			StreamCard sc = stream.getFirstCardWithID(id);
			stream.remove(id);
			sc.setBestNextReviewDate(StreamCard.formatTime(newTime));
			newTime += 1000;
			stream.add(sc);
			selectedCards.add(sc.toXML());
		}
		displayPanel.redraw();
		setCardContainerChangesMade(true);
	}

	/**
	 * Called when the user requests selected cards to be moved to a different
	 * compartment.
	 */
	public void onMoveCards() {
		int[] cardids = cardTablePanel.getSelectedCardIDs();
		// TODO: Move Cards for streams
		int scomp = ((BoxDisplayPanel) displayPanel).getSelectedCompartment();
		if (cardids.length < 1)
			return;
		if (scomp < 0)
			return;
		Box b = data.getBox();
		String s = "Move cards to partition (0-" + (b.getPartitionCount() + 1)
				+ "):";
		String choice = JOptionPane.showInputDialog(s, "0");
		int pchoice;
		try {
			pchoice = Integer.parseInt(choice);
		} catch (Exception nfe) {
			return;
		}
		pchoice--;
		int cchoice;
		if (pchoice == b.getPartitionCount()) {
			cchoice = (b.getPartitionCount() * 2) + 2;
		} else {
			cchoice = b.partitionNumberToCompartmentNumber(pchoice);
		}

		// Move cards.
		for (int c = 0; c < cardids.length; c++) {
			b.move(cardids[c], scomp, cchoice);
		}

		// Update displays
		if (b.getCompartmentCardCount(scomp) == 0) {
			displayPanel.onClick(0, 0);
		} else {
			redrawDisplayPanel();
			onSelectionChanged();
		}
		messagelabel.setText("Moved " + cardids.length
				+ " card(s) from partition "
				+ b.compartmentNumberToPartitionNumber(scomp) + " to "
				+ b.compartmentNumberToPartitionNumber(cchoice) + ".");
	}

	/**
	 * Called when the user requests selected cards to be removed.
	 */
	public void onRemoveCards() {

		int[] cardids = cardTablePanel.getSelectedCardIDs();

		int scomp = 0;
		if (data.getCardContainer() instanceof Box) {
			scomp = ((BoxDisplayPanel) displayPanel).getSelectedCompartment();
			if (cardids.length < 1)
				return;
			if (scomp < 0)
				return;
		}

		String s = Strings.get(67);
		int choice = JOptionPane.showConfirmDialog(this, s, Strings.get(68),
				JOptionPane.YES_NO_OPTION);
		if (choice != JOptionPane.YES_OPTION)
			return;

		// Remove cards.
		if (data.getCardContainer() instanceof Box) {
			for (int c = 0; c < cardids.length; c++) {
				data.getBox().remove(cardids[c], scomp);
			}
			if (data.getBox().getCompartmentCardCount(scomp) == 0) {
				displayPanel.onClick(0, 0);
			} else {
				redrawDisplayPanel();
				onSelectionChanged();
			}
		} else {
			for (int c = 0; c < cardids.length; c++) {
				data.getCardContainer().remove(cardids[c]);
			}
			displayPanel.onClick(0, 0);
		}

		// Update displays
		messagelabel.setText(Strings.get(69) + cardids.length + Strings.get(70)
				+ getCardContainerTypeLowerCase() + ".");
		summaryPanel.updateSummary();
	}

	/**
	 * Called when the user requests to edit a selected card.
	 */
	public void onEditCard() {
		int[] cardids = cardTablePanel.getSelectedCardIDs();

		if (data.getCardContainer() instanceof Box) {
			int scomp = ((BoxDisplayPanel) displayPanel)
					.getSelectedCompartment();
			if (cardids.length < 1)
				return;
			if (scomp < 0)
				return;
		}

		editCard(cardids[0]);
	}

	public void editCard(int cardid) {
		new editCardPanel(this, cardid);
		onSelectionChanged();
	}

	// Used by onEdit() to edit cards in the box while browsing.
	private class editCardPanel extends JDialog {

		private MetaCardDisplayPanel mceditpanel;

		int cardid;

		public editCardPanel(Component owner, int newcardid) {

			MetaCard mc = data.getCardSet().getMetaCard(newcardid);
			cardid = newcardid;

			getContentPane().setLayout(new BorderLayout());
			JButton b = new JButton(Strings.get(72));
			// TODO: this is a temporary workaround for getting font sizes to
			// editpanel without FontController
			mceditpanel = mc.getEditPanel( b,
					(Stream)AnkiStreamFrame.this.getData().getCardContainer());
			getContentPane().add(mceditpanel, BorderLayout.CENTER);

			// Control panel:
			JPanel p = new JPanel(new FlowLayout());
			p.setBorder(BorderFactory.createTitledBorder(""));
			b.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onSaveToCardSet(cardid, mceditpanel);
					dispose();
				}
			});
			p.add(b);
			b = new JButton(Strings.get(73));
			b.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					dispose();
				}
			});
			p.add(b);
			getContentPane().add(p, BorderLayout.SOUTH);

			setTitle(Strings.get(74));
			pack();
			setLocationRelativeTo(owner);
			setModal(true);
			setVisible(true);
		}

	} // end class editCardPanel

	public void onReshuffleCards() {
		// TODO: Shuffle for stream?
		int compartment = ((BoxDisplayPanel) displayPanel)
				.getSelectedCompartment();
		if (compartment == -1)
			return;
		data.getBox().shuffleCompartment(compartment);
		onSelectionChanged();
	}

	/**
	 * Prints some statistics about the current review in progress to the
	 * message label.
	 */
	public void showReviewStatusMessage() {

		CardContainer cc = data.getCardContainer();

		if (cc.getCurrentCardID() == -1)
			return; // not reviewing

		String msg = Strings.get(75) + cc.getReviewStatusString()
				+ Strings.get(76);
		messagelabel.setText(msg);
	}

	/**
	 * Enables or disables long term review menu item and button depending on if
	 * there are cards in the post-box review pile to be reviewed.
	 */
	private void enableOrDisableLongTermReviewControls() {
		// TODO: Remove long term review concept from stream interface.
		if (data.getCardContainer() instanceof Box) {
			int postboxccount = data.getBox().getCompartmentCardCount(
					(data.getBox().getPartitionCount() * 2) + 2);
			boolean enabled = false;
			if (postboxccount > 0)
				enabled = true;
			beginlongtermreviewbutton.setEnabled(enabled);
			reloadMenuBar();
		}
	}

	private MetaCardDisplayPanel getSizedMCDisplayPanel(int cardID) {
		MetaCardDisplayPanel mcdp = data.getCardSet().getMetaCard(cardID)
				.getDisplayPanel();
		mcdp.setQuestionFontSize(data.getCardContainer().getQuestionFontSize());
		mcdp.setAnswerFontSize(data.getCardContainer().getAnswerFontSize());
		return mcdp;
	}

	/**
	 * Helper method for review starting methods below.
	 */
	private void setUpReviewInterface() {
		int cardID = data.getCardContainer().getCurrentCardID();
		metacarddisplaypanel = getSizedMCDisplayPanel(cardID);

		getContentPane().remove(leftpanel);
		reviewpanel.add(metacarddisplaypanel, BorderLayout.CENTER);
		getContentPane().add(reviewpanel);
		metacarddisplaypanel.layoutPanel(savetocardsetbutton);
		redrawDisplayPanel(true);
		toggleUndo();
		showReviewStatusMessage();
		displayPanel.redraw();
		pack();
		repaint();
		flipbutton.requestFocus();
		startReviewTimer();
		currentphase = PHASE_REVIEW;

		reloadMenuBar();
	}

	/**
	 * Called when the user requests a long term review to begin.
	 */
	public void onBeginLongTermReview() {
		// Assume this is a box. Streams don't have "long term review"s.
		if (data.getCardContainer() instanceof Box) {
			return;
		}
		((Box) data.getCardContainer()).beginLongTermReview();
		setUpReviewInterface();
	}

	public void onBeginReview() {
		onBeginReview(-1, true);
	}

	public void onBeginReview(int beginAt) {
		onBeginReview(beginAt, true);
	}

	/**
	 * Called when the user requests a review to begin.
	 * 
	 * @param beginAt
	 *            the partition to begin at.
	 * @param endreviewfirst
	 *            Should the old review session (if there was one) be ended
	 *            before beginning the new one?
	 */
	public void onBeginReview(int beginAt, boolean endoldreview) {
		// Be sure to unselect the current compartment if we are browsing.
		// TODO: Unselect for streams too.
		if (data.getCardContainer() instanceof Box) {
			if (data.getBox().getCurrentPartitionNumber() != -1
					&& ((BoxDisplayPanel) displayPanel)
							.getSelectedCompartment() != -1) {
				displayPanel.onClick(0, 0);
			}
			data.getBox().beginReview(-1, endoldreview);
		} else {
			data.getCardContainer().beginReview();
		}

		if (data.getCardContainer().getCurrentCardID() == -1) {
			JOptionPane.showMessageDialog(null, Strings.get(77), Strings
					.get(78), JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		setUpReviewInterface();
	}

	/**
	 * Called when the user requests a review to end.
	 */
	public void onEndReview() {

		if (data.getCardContainer() instanceof Box) {
			((Box) data.getCardContainer()).endReview(false); // no reshuffle
		} else {
			data.getCardContainer().endReview();
		}
		reviewpanel.remove(metacarddisplaypanel);
		redrawDisplayPanel(true);
		reconfigurePanelsAfterReview(true);
	}

	/**
	 * Called when the user requests a card flip.
	 */
	public void onFlip() {
		stopReviewTimer();
		if (data.getCardContainer().getCurrentCardID() != -1) {
			metacarddisplaypanel.showAnswer();
			reviewcontrolpanel.ToggleControls();
			currentphase = PHASE_RESULT;
			rightbutton.setEnabled(true);
			wrongbutton.setEnabled(true);
			rightbutton.requestFocus();
		}
	}

	public void disableUndo() {
		undobutton.setEnabled(false);
	}

	public void enableUndo() {
		undobutton.setEnabled(true);
	}

	public void toggleUndo() {
		if (data.getCardContainer().canUndo()) {
			enableUndo();
		} else {
			disableUndo();
		}
	}

	// Updates the box summary panel.
	public void updateSummary() {
		summaryPanel.updateSummary();
	}

	/**
	 * Called when the user gets a card right.
	 */
	public void onRight() {

		// Don't do anything if no review is in progress.
		if (data.getCardContainer().getCurrentCardID() == -1) {
			return;
		}

		setCardContainerChangesMade(true); // The card container has changed.

		// Update the box stats and display.
		data.getCardContainer().right();
		int cardID = data.getCardContainer().getCurrentCardID();
		toggleUndo();
		reviewpanel.remove(metacarddisplaypanel);
		redrawDisplayPanel(true);

		// Reconfigure the interface if the review is done.
		if (cardID == -1) {
			reconfigurePanelsAfterReview();
			return;
		}

		showReviewStatusMessage();
		checkForAutoSave();

		// Toggle the review control panel.
		reviewcontrolpanel.ToggleControls();

		// Update the metacard display panel.
		metacarddisplaypanel = getSizedMCDisplayPanel(cardID);
		reviewpanel.add(metacarddisplaypanel, BorderLayout.CENTER);
		metacarddisplaypanel.layoutPanel(savetocardsetbutton);
		repaint();
		startReviewTimer();
		currentphase = PHASE_REVIEW;
		flipbutton.requestFocus();
	}

	/**
	 * Autosaves the stream if the autosave review count has been reached.
	 */
	private void checkForAutoSave() {
		if (!(data.getCardContainer() instanceof Stream)) {
			return;
		}
		Stream s = (Stream) data.getCardContainer();
		if (!s.getUseAutoSaveEvery()) {
			return;
		}
		String msg = messagelabel.getText();
		if (s.getReviewed() % s.getAutoSaveEvery() == 0) {
			if (onSave()) {
				setMessage(msg + " " + getCardContainerSavedMessage());
			} else {
				setMessage(msg + " " + Strings.get(50));
			}
			flipbutton.requestFocus();
		}
	}

	/**
	 * Called when the user gets a card wrong.
	 */
	public void onWrong() {

		// Don't do anything if no review is in progress.
		if (data.getCardContainer().getCurrentCardID() == -1) {
			return;
		}

		setCardContainerChangesMade(true); // The card container has changed.

		// Update the box stats and display.
		data.getCardContainer().wrong();
		int cardID = data.getCardContainer().getCurrentCardID();
		reviewpanel.remove(metacarddisplaypanel);
		toggleUndo();
		redrawDisplayPanel(true);
		savetocardsetbutton.setEnabled(false);

		// Reconfigure the interface if the review is done.
		if (cardID == -1) {
			reconfigurePanelsAfterReview();
			return;
		}

		showReviewStatusMessage();
		checkForAutoSave();

		// Toggle the review control panel.
		reviewcontrolpanel.ToggleControls();

		// Update the metacard display panel.
		metacarddisplaypanel = getSizedMCDisplayPanel(cardID);
		reviewpanel.add(metacarddisplaypanel, BorderLayout.CENTER);
		metacarddisplaypanel.layoutPanel(savetocardsetbutton);
		repaint();
		startReviewTimer();
		currentphase = PHASE_REVIEW;
		flipbutton.requestFocus();
	}

	/**
	 * Called by onRight and onWrong and onEndReview.
	 */
	private void reconfigurePanelsAfterReview() {
		reconfigurePanelsAfterReview(false);
	}

	/**
	 * Called by onRight and onWrong and onEndReview.
	 */
	private void reconfigurePanelsAfterReview(boolean doNotSave) {
		// Switch back to review phase controls for the next review:
		if (currentphase == PHASE_RESULT)
			reviewcontrolpanel.ToggleControls();
		getContentPane().remove(reviewpanel);

		summaryPanel.updateSummary();
		enableOrDisableLongTermReviewControls();
		CardContainer cc = data.getCardContainer();
		String msg = Strings.get(79) + cc.getReviewStatusString()
				+ Strings.get(80);
		boolean saveafterreview = true;
		if (data.getCardContainer() instanceof Stream) {
			saveafterreview = ((Stream) data.getCardContainer())
					.getAutoSaveAfterReview();
		}
		if (saveafterreview && !doNotSave) {
			if (onSave()) {
				msg += " " + getCardContainerSavedMessage();
			} else {
				msg += " " + Strings.get(50);
			}
		}
		messagelabel.setText(msg);
		currentphase = PHASE_BROWSING;
		reloadMenuBar();
		// TODO: next line causes unwanted frame resize
		displayPanel.onClick(0, 0); // Unselect all compartments.
		getContentPane().add(leftpanel, BorderLayout.CENTER);
		pack();
		repaint();
		redrawDisplayPanel(true);
		beginreviewbutton.requestFocus(); //Highlight the start review button.
	}

	/**
	 * Called when the user requests an undo of the last action.
	 */
	public void onUndo() {

		if (!data.getCardContainer().Undo())
			return;

		int cardID = data.getCardContainer().getCurrentCardID();

		if (currentphase == PHASE_BROWSING) {
			// If we are undoing back into a review session from browsing,
			// set up the review panels and everything.
			getContentPane().remove(leftpanel);
			getContentPane().add(reviewpanel);
		} else if (currentphase == PHASE_RESULT) {
			// We are undoing from review or result phase.
			reviewcontrolpanel.ToggleControls();
			reviewpanel.remove(metacarddisplaypanel);
		} else if (currentphase == PHASE_REVIEW) {
			// We are undoing from review or result phase.
			reviewpanel.remove(metacarddisplaypanel);
		}

		// Switch back to the review phase.
		if (displayPanel instanceof StreamDisplayPanel) {
			// Workaround a panel-not-redrawing-properly bug.
			((StreamDisplayPanel) displayPanel).adjustRangeToZoomSlider();
		}
		redrawDisplayPanel(true);
		metacarddisplaypanel = getSizedMCDisplayPanel(cardID);
		reviewpanel.add(metacarddisplaypanel, BorderLayout.CENTER);
		metacarddisplaypanel.layoutPanel(savetocardsetbutton);
		repaint();
		startReviewTimer();
		currentphase = PHASE_REVIEW;
		toggleUndo();
		showReviewStatusMessage();
		flipbutton.requestFocus();
	}

	/**
	 * Called when the user requests the current metacard be saved to the card
	 * set during box browsing.
	 */
	public void onSaveToCardSet(int cardid, MetaCardDisplayPanel mcdp) {
		MetaCard mc = data.getCardSet().getMetaCard(cardid);
		String newcontents = mcdp.getEditedContent();
		mc.setContents(newcontents);
		onSaveCardSet();
	}

	/**
	 * Called when the user requests the current metacard be saved to the card
	 * set during review.
	 */
	public void onSaveToCardSet() {
		int cardid = data.getCardContainer().getCurrentCardID();
		MetaCard mc = data.getCardSet().getMetaCard(cardid);
		String newcontents = metacarddisplaypanel.getEditedContent();
		mc.setContents(newcontents);
		onSaveCardSet();
	}

	/**
	 * Panel for giving the user review time controls:
	 */
	private class ReviewControlPanel extends JPanel {

		public ReviewControlPanel() {
			setPreferredSize(new Dimension(320, 80));
			setBorder(BorderFactory.createTitledBorder(""));

			// Lay out controls.
			// preflip phase
			setLayout(new GridLayout(0, 1));
			rpupperpanel = new JPanel(new FlowLayout());
			rpupperpanel.add(savetocardsetbutton);
			rpupperpanel.add(endreviewbutton);
			add(rpupperpanel);
			rplowerpanel = new JPanel(new FlowLayout());
			rplowerpanel.add(flipbutton);
			rplowerpanel.add(timelabel);
			rplowerpanel.add(undobutton);
			add(rplowerpanel);
			savetocardsetbutton.setEnabled(false);

		}

		// Toggles the preflip/postflip controls.
		public void ToggleControls() {
			if (currentphase == PHASE_REVIEW) {
				currentphase = PHASE_RESULT;
				rplowerpanel.removeAll();
				rplowerpanel.add(rightbutton);
				rplowerpanel.add(wrongbutton);
				rplowerpanel.add(timelabel);
				rplowerpanel.add(undobutton);
				savetocardsetbutton.setEnabled(true);
				// No redraw bug workaround:
				setVisible(false);
				setVisible(true);
			} else {
				currentphase = PHASE_REVIEW;
				rplowerpanel.removeAll();
				rplowerpanel.add(flipbutton);
				rplowerpanel.add(timelabel);
				rplowerpanel.add(undobutton);
				savetocardsetbutton.setEnabled(false);
			}
			reloadMenuBar();
		}

	} // end class ReviewControlPanel

	/**
	 * A panel for finding cards. <br>
	 * <br>
	 * Part of the "Anki Box" project, a tool aiding long-term flash card
	 * drilling as outlined in James Heisig's "Remembering the Kana".
	 * 
	 * @author Bishop Wilkins
	 * @version 1.0
	 */
	private class FindCardPanel extends JDialog {

		// The preferences settings indicated on this form.
		private Vector results; // of the most recent word search

		private int nextresult; // which result does the user want to see.

		// Controls
		JTextField searchstringfield;

		JLabel resultsmessagelabel;

		JButton findbutton;

		JButton findnextbutton;

		JButton okbutton;

		/**
		 * Creates and shows a new Edit Preferences dialog using the specified
		 * preferences.
		 */
		public FindCardPanel(JFrame owner) {

			// Set the owner.
			super(owner);

			results = null;
			nextresult = 0;

			// Layout the preferences dialog.
			getContentPane().setLayout(new GridLayout(0, 1));
			JPanel p = new JPanel(new FlowLayout());
			p.add(new JLabel(Strings.get(81)));
			getContentPane().add(p);
			p = new JPanel(new FlowLayout());
			searchstringfield = new JTextField("", 17);
			p.add(searchstringfield);
			getContentPane().add(p);
			p = new JPanel(new FlowLayout());
			resultsmessagelabel = new JLabel("");
			p.add(resultsmessagelabel);
			getContentPane().add(p);
			p = new JPanel(new FlowLayout());
			findbutton = new JButton(Strings.get(82));
			findbutton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onFind();
				}
			});
			p.add(findbutton);
			findnextbutton = new JButton(Strings.get(83));
			findnextbutton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					onFindNext();
				}
			});
			findnextbutton.setEnabled(false);
			p.add(findnextbutton);
			okbutton = new JButton(Strings.get(84));
			okbutton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					dispose();
				}
			});
			p.add(okbutton);
			getContentPane().add(p);

			setTitle(Strings.get(85));
			pack();
			setLocationRelativeTo(owner);
			setModal(true);
			setVisible(true);
			searchstringfield.requestFocus();
		}

		private void onFind() {
			String s = searchstringfield.getText();
			if (s.length() == 0)
				return;
			results = data.getCardContainer().findCardsContainingString(s,
					data.getCardSet());
			s = results.size() + Strings.get(210);
			resultsmessagelabel.setText(s);
			if (results.size() > 1) {
				findnextbutton.setEnabled(true);
			} else {
				findnextbutton.setEnabled(false);
			}

			if (results.size() > 0) {
				nextresult = 0;
				highlightFoundCard();
			}
		}

		private void onFindNext() {
			nextresult++;
			if (nextresult >= results.size()) {
				nextresult = 0;
			}
			if (results.size() > 0) {
				highlightFoundCard();
			}
		}

		private void highlightFoundCard() {
			AnkiStreamFrame.this.highlightFoundCard(results, nextresult);
		}

	} // end FindCardPanel

	/**
	 * Highlights the card indicated by entry nextresult in Vector results,
	 * which should have been returned from Box.findCardsContainingString().
	 */
	public void highlightFoundCard(Vector results, int nextresult) {

		int selectthisid = 0;

		// Select Compartment
		if (data.getCardContainer() instanceof Box) {
			int[] r = (int[]) results.get(nextresult);
			selectthisid = r[1];
			if (!((BoxDisplayPanel) displayPanel).selectCompartment(r[0])) {
				return;
			}
		} else {
			int r = ((Integer) results.get(nextresult)).intValue();
			((StreamDisplayPanel) displayPanel).selectCardWithID(r);
			selectthisid = r;
		}

		onSelectionChanged();

		cardTablePanel.selectCard(selectthisid);
	}

	/**
	 * Handles typical Anki Box key presses.
	 */
	private class AnkiBoxKeyListener extends KeyAdapter {

		public void keyReleased(KeyEvent ke) {

			if (currentphase == PHASE_REVIEW) {
				switch (ke.getKeyCode()) {
				case KeyEvent.VK_F:
				case KeyEvent.VK_R:
				case KeyEvent.VK_W:
					flipbutton.doClick();
					break;
				case KeyEvent.VK_U:
					undobutton.doClick();
				}
			} else if (currentphase == PHASE_RESULT) {
				switch (ke.getKeyCode()) {
				case KeyEvent.VK_R:
					rightbutton.doClick();
					break;
				case KeyEvent.VK_W:
					wrongbutton.doClick();
					break;
				}
			}

		} // end keyReleased

	} // end class AnkiBoxKeyListener

	/**
	 * Panel for show the about dialog:
	 */
	private class AboutDialog extends JDialog {

		public AboutDialog() {

			super(AnkiStreamFrame.this);

			// Layout Controls.
			setTitle(Strings.get(32));
			getContentPane().setLayout(
					new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));

			JPanel p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			JLabel l = new JLabel(Strings.get(3));
			l.setFont(l.getFont().deriveFont(Font.BOLD));
			p.add(l);
			add(p);
			p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			l = new JLabel(" " + Strings.get(211) + " ");
			p.add(l);
			add(p);
			p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			l = new JLabel(Strings.get(212));
			p.add(l);
			add(p);
			p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			l = new JLabel(Strings.get(213));
			p.add(l);
			add(p);
			p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			l = new JLabel(Strings.get(214));
			// l.setFont(FontController.getSizedFont(12));
			l.setFont(l.getFont().deriveFont(12));
			p.add(l);
			add(p);
			p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			l = new JLabel(Strings.get(215));
			// l.setFont(FontController.getSizedFont(12));
			l.setFont(l.getFont().deriveFont(12));
			p.add(l);
			add(p);

			p = new JPanel(new FlowLayout(FlowLayout.CENTER, 1, 1));
			JButton b = new JButton(Strings.get(84));
			b.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					dispose();
				}
			});
			p.add(b);
			add(p);

			pack();
			setLocationRelativeTo(AnkiStreamFrame.this);
			setModal(true);
			setVisible(true);
		}
	}

	public String getCardContainerType() {
		if (data.getCardContainer() instanceof Stream) {
			return Strings.get(4);
		} else {
			return Strings.get(7);
		}
	}

	public String getCardContainerTypeLowerCase() {
		return getCardContainerType().toLowerCase();
	}

} // end class AnkiBoxFrame
