package ankistream;

import java.awt.Component;
import java.awt.Toolkit;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.KeyboardFocusManager;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.InputEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;
import java.util.Set;
import java.util.TreeSet;
import java.util.Collections;
import java.util.Random;
import java.util.NoSuchElementException;
import java.util.zip.DataFormatException;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** A type of metacard that holds display information for a basic, two sided
  * flash card.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class BasicCard extends MetaCard {
	
	public String question;
	public String answer;
	
	public BasicCard( ) {
		super( );
	}
		
	public BasicCard(	int newid,
						String newquestion,
						String newanswer ) {
		question = newquestion;
		answer = newanswer;
		xml = new XMLElement( "metacard" );
		setID( newid );
		setContents( question + "\t" + answer );
		setType( "basic" );
	}
	
	public void initializeFromCData( ) {		
		String cdata = xml.getContent();
		String[] tokens = cdata.split( "\t" );
		if ( tokens.length >= 2 ) {
			question = tokens[0];
			answer = cdata.substring( cdata.indexOf( "\t" )+1 );			
		} else {
			question = "Error loading flash card data.";
			answer = "Error loading flash card data.";
		}		
	}
	
	/** Gets a short string representing this card's "question" (front side)
	  * for use in card browsing tables and such.
	 */
	public String getQuestion( ) { return question; }

	/** Gets a short string representing this card's "answer" (back side)
	  * for use in card browsing tables and such.
	 */
	public String getAnswer( ) { return answer; }

	public MetaCardDisplayPanel getDisplayPanel( ) {		
		return new bcDisplayPanel();		
	}

	public MetaCardDisplayPanel getDisplayPanel( 
			float questionFontSize, float answerFontSize ) {		
		return new bcDisplayPanel( questionFontSize, answerFontSize );		
	}

	public MetaCardDisplayPanel getEditPanel( ) {
		return new bcEditPanel();
	}
	
	public MetaCardDisplayPanel getEditPanel( Component c ) {
		return new bcEditPanel( c );
	}
	
	public MetaCardDisplayPanel getEditPanel( Component c, Stream s ) {
		return new bcEditPanel( c, s );
	}
	
	private class bcDisplayPanel extends MetaCardDisplayPanel {
		
		public static final int WIDTH_POINTS = 384;
		public static final int HEIGHT_POINTS = 144;
		
		protected JTextArea questionarea;
		protected JTextArea answerarea;
		protected JButton savecardtocardsetbutton;
		
		protected float questionFontSize;
		protected float answerFontSize;
		
		protected Component focusmeafteranswer;
				
		public bcDisplayPanel( ) {
			//setQuestionFontSize(FontController.getUnicodeFont().getSize());
			//setAnswerFontSize(FontController.getUnicodeFont().getSize());
		} // end bcDisplayPanel constructor
		
		public bcDisplayPanel(	float newQuestionFontSize, 
								float newAnswerFontSize ) {
			setQuestionFontSize( newQuestionFontSize );
			setAnswerFontSize( newAnswerFontSize );
		}
		
		public void setQuestionFontSize( float newVal ) {
			questionFontSize = newVal;
		}
		
		public void setAnswerFontSize( float newVal ) {
			answerFontSize = newVal;
		}
		
		public void layoutPanel( ) {
			layoutPanel( null );
		}
		
		public void layoutPanel( Component newfocusafteranswer ) {
			
			setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
			setBorder( BorderFactory.createTitledBorder( "" ) );
		
			questionarea = new JTextArea( 
					calculateHeightForFontSize(questionFontSize), 
					calculateWidthForFontSize(questionFontSize) );
			answerarea = new JTextArea( 
					calculateHeightForFontSize(answerFontSize), 
					calculateWidthForFontSize(answerFontSize) );
			if ( newfocusafteranswer == null ) {
				newfocusafteranswer = questionarea;
			}
			focusmeafteranswer = newfocusafteranswer;

			JPanel p = new JPanel( new FlowLayout( FlowLayout.CENTER,1,1 ) );
			questionarea.setWrapStyleWord( true );
			questionarea.setLineWrap( true );
			//questionarea.setFont(FontController.getSizedFont(questionFontSize));
			questionarea.setFont(
					questionarea.getFont().deriveFont(questionFontSize));
			questionarea.setText( question );
			questionarea.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,null);
			questionarea.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,null); 

			p.add( new JScrollPane( questionarea ) );
			add( p );

			p = new JPanel( new FlowLayout( FlowLayout.CENTER,1,1 ) );
			answerarea.setWrapStyleWord( true );
			answerarea.setLineWrap( true );
			//answerarea.setFont(FontController.getSizedFont(answerFontSize));
			answerarea.setFont(
					answerarea.getFont().deriveFont(answerFontSize));
			answerarea.setText( " " );			
			answerarea.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,null);
			answerarea.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,null); 

			p.add( new JScrollPane( answerarea ) );
			add( p );						
			
			addFocusListener( new FocusAdapter() {
				public void focusGained(FocusEvent fe) {
					questionarea.requestFocus();
				}
			});
		}
		
		protected int calculateWidthForFontSize( float fontSize ) {
			return Math.max(1,(int)((float)WIDTH_POINTS / fontSize));
		}

		protected int calculateHeightForFontSize( float fontSize ) {
			return Math.max(1,(int)((float)HEIGHT_POINTS / fontSize));
		}

		public void showQuestion( ) {
			questionarea.setText( question );
			answerarea.setText( " " );
		}

		public void showAnswer( ) {
			questionarea.setText( question );
			answerarea.setText( answer );
		}

		public String getEditedContent( ) {
			String rval = questionarea.getText();
			rval += "\t";
			rval += answerarea.getText();
			return rval;
		}
		
	} // end class bcDisplayPanel

	private class bcEditPanel extends bcDisplayPanel {
						
		public bcEditPanel( ) {
			super( );
			layoutPanel( );
		} // end bcDisplayPanel constructor
		
		public bcEditPanel( Component newfocusafteranswer ) {
			super( );
			layoutPanel( newfocusafteranswer );
		}

		public bcEditPanel( Component newfocusafteranswer, Stream s ) {
			super( );
			questionFontSize = s.getQuestionFontSize();
			answerFontSize = s.getAnswerFontSize();
			layoutPanel( newfocusafteranswer );
		}

		public void layoutPanel( ) {
			layoutPanel( null );
		}
				
		public void layoutPanel( Component newfocusafteranswer ) {
			setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
			setBorder( BorderFactory.createTitledBorder( "" ) );
		
			questionarea = new JTextArea( 
					calculateHeightForFontSize(questionFontSize), 
					calculateWidthForFontSize(questionFontSize) );
			answerarea = new JTextArea( 
					calculateHeightForFontSize(answerFontSize), 
					calculateWidthForFontSize(answerFontSize) );
			if ( newfocusafteranswer == null ) {
				newfocusafteranswer = questionarea;
			}
			focusmeafteranswer = newfocusafteranswer;

			JPanel p = new JPanel( new FlowLayout( FlowLayout.CENTER,1,1 ) );
			p.add( new JLabel(Strings.get(168)) );
			add( p );

			p = new JPanel( new FlowLayout( FlowLayout.CENTER,1,1 ) );
			questionarea.setWrapStyleWord( true );
			questionarea.setLineWrap( true );
			//questionarea.setFont(FontController.getSizedFont(questionFontSize));
			questionarea.setFont(
					questionarea.getFont().deriveFont(questionFontSize));
			questionarea.setText( question );
			questionarea.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,null);
			questionarea.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,null); 

			p.add( new JScrollPane( questionarea ) );
			add( p );

			p = new JPanel( new FlowLayout( FlowLayout.CENTER,1,1 ) );
			p.add( new JLabel(Strings.get(169)) );
			add( p );

			p = new JPanel( new FlowLayout( FlowLayout.CENTER,1,1 ) );
			answerarea.setWrapStyleWord( true );
			answerarea.setLineWrap( true );
			//answerarea.setFont(FontController.getSizedFont(answerFontSize));
			answerarea.setFont(
					answerarea.getFont().deriveFont(answerFontSize));
			answerarea.setText( answer );			
			answerarea.setFocusTraversalKeys(KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS,null);
			answerarea.setFocusTraversalKeys(KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS,null); 

			p.add( new JScrollPane( answerarea ) );
			add( p );						
			
			addFocusListener( new FocusAdapter() {
				public void focusGained(FocusEvent fe) {
					questionarea.requestFocus();
				}
			});
		}

//TODO: don't need this anymore i don't think...
/*		
		public void layoutPanel( ) {
			setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
			setBorder( BorderFactory.createTitledBorder( "" ) );
		
			JPanel p = new JPanel( new FlowLayout( ) );
			p.add( new JLabel(Strings.get(168)) );
			add( p );
			p = new JPanel( new FlowLayout() );
			questionarea = new JTextArea( 11, 28 );
			questionarea.setWrapStyleWord( true );
			questionarea.setLineWrap( true );
			questionarea.setText( question );			
			p.add( new JScrollPane( questionarea ) );
			add( p );			

			p = new JPanel( new FlowLayout( ) );
			p.add( new JLabel(Strings.get(169)) );
			add( p );
			p = new JPanel( new FlowLayout() );
			answerarea = new JTextArea( 11, 28 );
			answerarea.setWrapStyleWord( true );
			answerarea.setLineWrap( true );
			answerarea.setText( answer );			
			p.add( new JScrollPane( answerarea ) );
			add( p );						
		}
*/	
	} // end class bcEditPanel

} // end class BasicCard
