package ankistream;

import java.awt.Graphics;
import java.util.Vector;

/** Defines what the main card display panels on the right side of the Anki
  * Stream application must provide.  Implemented by box and stream displays.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public interface CardContainerDisplayPanel {
	
	/** Begins invoking specified AnkiStreamFrame's onSelectionChanged()
	  * method whenever a new card range is selected/deselected. 
	 */
	public void notifyMeOnSelection( AnkiStreamFrame newowner );
	
	/** Redraws the graphical representation of the box.
	 */
	public void redraw( );
	
	/** Called whenever the user clicks the mouse on the display panel.
	  * Tests whether the user clicked on a card and if so selects that
	  * card area unless a review is underway.
	  * @param x x position the mouse was clicked at. Relative to this panel.
	  * @param y y position the mouse was clicked at. Relative to this panel.
	 */
	public void onClick( int x, int y );
	
	// TODO: some kind of get and set selected card range mechanismm
	/** Returns the id of the selected card, or -1 if no card is selected.
	 */
	public int getSelectedCardID();
	
	/** Gets a vector of IXMLElements representing the cards around and 
	  * including the selected card to browse.
	 */
	public Vector getCardRange();
	
	public void setCardContainer( CardContainer cc );
	/** Gives this renderer an interface to access its card container.
	 */	

} // end class CardContainerDisplayPanel
