package ankistream;
import java.io.File;
import java.util.Random;
import java.util.Vector;
import java.util.List;
import java.util.Iterator;
import java.util.Collections;
import java.util.Comparator;
import java.awt.Toolkit;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane; 
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.JTable;
import javax.swing.table.*;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.Timer;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** A panel containing a table of cards for use when browsing a box.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class CardTablePanel 
	extends JPanel implements CommonCardTablePanel {

	// Vector of IXMLElements representing cards to be displayed in this table. 
	private Vector cards;
	private JScrollPane scrollpane;
	private JTable cardtable;
	private CardTableModel cardtablemodel;
	private CardSet cset; // A reference to the card set for card names.
	private AnkiStreamFrame ownerframe;
	
	/** The constructor takes a list of IXMLElements representing cards to be
	  * displayed in this table, and a card set to use to display information
	  * about them.
	 */
	public CardTablePanel(	Vector newcards, 
							CardSet newcset, 
							AnkiStreamFrame newownerframe ) {		
		super();
		
		cset = newcset;
		cards = newcards;
		ownerframe = newownerframe;
		
		// Create the table model and populate it with the specified cards.
		createTableModelAndTable();
	}
	
	private void createTableModelAndTable() {
		
		int cardcount = cards.size();
		String[][] tabledata = new String[cardcount][4];
		String[] columnNames = new String[4];
		columnNames[0] = "Card";
		columnNames[1] = "Streak";
		columnNames[2] = "Total Rights";
		columnNames[3] = "Last Wrong";
		Card curcard;
		MetaCard curmetacard;
		for ( int c = 0; c < cards.size(); c++ ) {
			curcard = new Card( (IXMLElement)cards.get( c ) );
			curmetacard = cset.getMetaCard( curcard.getID() );
			tabledata[c][0] = curmetacard.getQuestion();
			tabledata[c][1] = Integer.toString( curcard.getStreak() );
			String totalrights = Integer.toString(curcard.getLifetimeRights());
			totalrights += " out of ";
			totalrights += 
				(curcard.getLifetimeRights() + curcard.getLifetimeWrongs());
			tabledata[c][2] = totalrights;
			tabledata[c][3] = curcard.getLastWrongDate().split( " " )[0];
		}
		
		cardtablemodel = new CardTableModel( tabledata, columnNames );
		
		// Create the table panel using the prepared table model.
		cardtable = new JTable( cardtablemodel );
		cardtable.setAutoResizeMode( JTable.AUTO_RESIZE_ALL_COLUMNS );
		
		TableColumnModel tcm = cardtable.getColumnModel();
		tcm.getColumn( 0 ).setPreferredWidth( 80 );
		tcm.getColumn( 1 ).setPreferredWidth( 30 );
		tcm.getColumn( 2 ).setPreferredWidth( 60 );
		tcm.getColumn( 3 ).setPreferredWidth( 60 );	
				
		// Layout everything on the panel.
		if ( scrollpane != null ) { remove( scrollpane ); }
		scrollpane = new JScrollPane( cardtable );
		setLayout( new GridLayout() );
		cardtable.setPreferredScrollableViewportSize( 
			new Dimension( 300, 120 ) );
		add( scrollpane );
		ownerframe.pack();

		// Add listener for reordering the panel based on column heading
		final JTableHeader th = cardtable.getTableHeader( ); 
		th.addMouseListener( new MouseAdapter( ) { 
			public void mousePressed( MouseEvent e ) { 
			  TableColumnModel cm = th.getColumnModel( ); 
			  int col = cm.getColumnIndexAtX( e.getX( ) ); 
			  if (col != -1) { 
				  onColumnHeaderClicked( col ); 
			  } 
			} 
		});
		
		// Add listener for card selection.
		ListSelectionModel rowSM = cardtable.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
		        if (e.getValueIsAdjusting()) return;
		
		        ListSelectionModel lsm =
		            (ListSelectionModel)e.getSource();
		        if (lsm.isSelectionEmpty()) {

		        } else {
		            ownerframe.onCardTableSelectionChanged();
		        }
		    }
		});
	}
	
	/** Gets a vector of Integers representing the ids of selected cards.
	 */
	public int[] getSelectedCardIDs() {
		int[] rval;
		int[] srows = cardtable.getSelectedRows();
		rval = new int[srows.length]; 
		Card curcard;
		for ( int c = 0; c < srows.length; c++ ) {
			curcard = new Card((IXMLElement)cards.get( srows[c] ));
			rval[c] = curcard.getID( );
		}
		return rval;
	}
	
	// Comparators for reordering the rows in the table:
	// Compare cards by question name.
	private class questioncomparator implements Comparator {
		public int compare( Object o1, Object o2 ) {
			Card c1 = new Card( (IXMLElement)o1 );
			Card c2 = new Card( (IXMLElement)o2 );
			MetaCard mc1 = cset.getMetaCard( c1.getID() );
			MetaCard mc2 = cset.getMetaCard( c2.getID() );
			String s1 = mc1.getQuestion();
			String s2 = mc2.getQuestion();
			return s1.compareToIgnoreCase( s2 );
		}
	}

	// Compare cards by streak.
	private class streakcomparator implements Comparator {
		public int compare( Object o1, Object o2 ) {
			Card c1 = new Card( (IXMLElement)o1 );
			Card c2 = new Card( (IXMLElement)o2 );
			return c1.getStreak() - c2.getStreak();
		}
	}

	// Compare cards by total rights.
	private class totalrightscomparator implements Comparator {
		public int compare( Object o1, Object o2 ) {
			Card c1 = new Card( (IXMLElement)o1 );
			Card c2 = new Card( (IXMLElement)o2 );
			String s1 = Integer.toString(c1.getLifetimeRights());
			s1 += " out of ";
			s1 += 
				(c1.getLifetimeRights() + c1.getLifetimeWrongs());
			String s2 = Integer.toString(c2.getLifetimeRights());
			s2 += " out of ";
			s2 += 
				(c2.getLifetimeRights() + c2.getLifetimeWrongs());
			return s1.compareToIgnoreCase( s2 );
		}
	}

	// Compare cards by last wrong date.
	private class lastwrongcomparator implements Comparator {
		public int compare( Object o1, Object o2 ) {
			Card c1 = new Card( (IXMLElement)o1 );
			Card c2 = new Card( (IXMLElement)o2 );
			String s1 = c1.getLastWrongDate();
			String s2 = c2.getLastWrongDate();
			return s2.compareToIgnoreCase( s1 );
		}
	}
	
	/** Invoked when the user clicks a column header to reorder the columns.
	 */
	private void onColumnHeaderClicked( int col ) {
		switch( col ) {
			case 1: {
				// Reorder by streak. 
				Collections.sort( cards, new streakcomparator() );
				break;
			}
			case 2: {
				// Reorder by total rights. 
				Collections.sort( cards, new totalrightscomparator() );
				break;
			}
			case 3: {
				// Reorder by last wrong date. 
				Collections.sort( cards, new lastwrongcomparator() );
				break;
			}
			default: {
				// Reorder by card question. 
				Collections.sort( cards, new questioncomparator() );
				break;
			}
		}
		createTableModelAndTable();
		ownerframe.onCardTableSelectionChanged( );
	}
	
	// TODO: The user finding a card, then reordering the card list, then
	// finding next could screw this up.
	public boolean selectCard( int cardid ) {
		// Find the card with the given id in the cards vector.
		Card ccard;
		for (int c = 0; c < cards.size(); c++ ) {
			ccard = new Card( (IXMLElement)cards.get(c) );
			if (ccard.getID() == cardid) {
				// Scroll to selected item.
				cardtable.setRowSelectionInterval( c, c );
				Rectangle rect = cardtable.getCellRect( c, 0, false ); 
				scrollpane.getVerticalScrollBar().setValue( rect.y );
				repaint();
				return true;
			}
		}
		return false;
	}
	
	private class CardTableModel extends AbstractTableModel {
		
		private String[][] data;
		private String[] columns;

		public int getColumnCount() { return columns.length; }
		public int getRowCount() { return data.length; }
		
		public CardTableModel( String[][] newdata, String[] newcolumns ) {
			data = newdata;
			columns = newcolumns;
		}
		
		public Object getValueAt( int row, int col ) {
			return data[row][col];
		}
		
		public String getColumnName( int column ) {
			return columns[column];
		}
		
		public boolean isCellEditable( int row, int col ) {
			return false;
		}
	}
		
}