package ankistream;

import java.text.SimpleDateFormat;
import java.util.Date;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** Interface describing the common functions of (Box) Cards and StreamCards.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
**/

public interface CommonCard {
	
	/** Gets the xml element of this card.  Should only be called by the
	  * Anki Stream framework when a new card is being created. */
	public IXMLElement toXML( );
	
	/** Gets the ID of this card. */
	public int getID();
	
	/** Gets the last review date of this card. */
	public String getLastReviewDate( );

	/** Gets the last date this card was missed in review. */
	public String getLastWrongDate( );

	/** Returns a positive number representing the number of times this card
	  * has been correctly identified in review, or a negative number
	  * representing the number of times it has been missed. */
	public int getStreak( );

	/** Gets the number of times this card has been correctly identified. */
	public int getLifetimeRights( );

	/** Gets the number of times this card has been missed. */
	public int getLifetimeWrongs( );

	/** Gets the creation date of this card. */
	public String getCreationDate();
		
	/** Sets the ID of this card. */
	public void setID( int newID );

	/** Gets the date. */
	public String getDateString();
}