package ankistream;

import java.io.File;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;
import java.util.Iterator;
import java.util.Random;
import java.awt.Toolkit;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem; 
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.XMLWriter;
import net.n3.nanoxml.XMLParserFactory;
import net.n3.nanoxml.StdXMLReader;

/** Tools for importing and exporting between flash card file formats.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class Converters {

	public static InputStreamReader isr;
	public static StringReader sr;
	public static BufferedReader br;
	public static OutputStreamWriter osw;
	
	/**  Adds readings from a readings file, one reading per line in this form:
	  * [character]\t[compound]\t[character reading]\t[compound reading[imi]]
	  * (also first line of the reading file is two unicode doublewidth spaces.
	  * followed by a newline.)
	  * ...to the cardset in the given stream file.  Also creates a new card
	  * list with a new card for each of the readings in the given file.
	 */
	public static void AddReadings( String readingsfile, 
									String cardsetfile, 
									String cardoutputfile ) throws IOException {
		
		AnkiStreamData asd = AnkiStreamData.loadDataFromFile(cardsetfile);
		
		CardSet cs = asd.getCardSet();

		initializestreams( readingsfile, cardoutputfile );
		osw.write("\n");
		FileOutputStream skippedfile;
		OutputStreamWriter skippedosw;
		try {
			skippedfile = new FileOutputStream( "skipped.txt" );
			skippedosw = new OutputStreamWriter( skippedfile, "UNICODE" );
		} catch ( Exception e ) {
			return;
		}

		// Count the highest 10000 (kun-yomi) id and 20000 (on-yomi) id
		int imax = cs.getNextAvailableID();
		System.out.print("imax:"+imax+"\n");
		int c = 0;
		int highestonyomi = 10000;
		int highestkunyomi = 20000;
		int tbdcount = 0;
		int k_replaced = 0, k_inserted = 0, k_ignored = 0, k_notfound = 0;
		int i;
		for (i = 1; i < imax; i++ ) {
			
			MetaCard mc = cs.getMetaCard(i);
			if (mc == null) {
				if (i > 10000 && i < 20000) { highestonyomi = i-1; }
				if (i > 20000) {highestkunyomi = i-1;}
				i += 6000;
				while (mc == null) {
					i+=3;
					mc = cs.getMetaCard(i);
				}
				c = 1;
			} else {
				if ((mc.getAnswer()+mc.getQuestion()).contains("TBD")) {
					tbdcount++;
				}
				c++;
			}
		}
		if (i > 20000) {highestkunyomi = i-1;}
		System.out.print( "highestonyomi: " + highestonyomi + "\n" );
		System.out.print( "highestkunyomi: " + highestkunyomi + "\n" );
		System.out.print( "TBD count: " + tbdcount + "\n" );
		highestonyomi++;
		highestkunyomi++;
		
        String newc = "        <card creationdate=\"now\" lastreviewdate=\"never reviewed\" bestnextreviewdate=\"now\" lasthundredhistory=\"\" lastwrongdate=\"never missed\" lifetimerights=\"0\" lifetimewrongs=\"0\" averageforgetperiod=\"never missed\" id=\"";
        String newcend = "\"/>\n";

        String kanji, compound, yomi, compoundyomi;
		String line = br.readLine().trim();;
		String blanks = line;
		while ( true ) {
			
			boolean kunyomi = false;

			// For each line,
			line = br.readLine();
			if ( line == null ) break;
			
			// Seek to the first tab.  Pull out everything before it as item 0
			int mark1 = line.indexOf( "." );
			if ( mark1 == -1 ) {
				//add blank lines to stream card output file on blank lines.
				osw.write("\n");
				continue;
			}
			compound = line.substring(0, mark1);
			
			// Continue, pulling out each token.
			line = line.substring( mark1+1 ).trim();
			mark1 = line.indexOf( "." );
			if ( mark1 == -1 ) continue;
			kanji = line.substring(0,mark1);

			String mcQuestion, mcAnswer;
			// Continue, pulling out each token.
			line = line.substring( mark1+1 ).trim();
			mark1 = line.indexOf( "." );
			if ( mark1 == -1 ) {
				// It's a kunyomi card.
				kunyomi = true;
				yomi = line;
				compoundyomi = "";
				mcQuestion = compound;
				mcAnswer = yomi + blanks;

			} else {
				yomi = line.substring(0,mark1);
				// Continue, pulling out each token.
				line = line.substring( mark1+1 ).trim();
				compoundyomi = line;
				mcQuestion = kanji + blanks + compound;
				mcAnswer = yomi + blanks + compoundyomi + blanks;
			}


			// Find metacard with this kanji.
			boolean opcomplete = false;
			int finalID = 0;			
			boolean founddupe = false;
			String findme = kanji + blanks;
			String orfindme = "TBD:" + kanji;
			if (kunyomi) { // Search for duplicate kunyomi readings.
				System.out.print("Searching for \""+compound+"."+yomi+"\"... " );
				for (i = 1; i < cs.getNextAvailableID(); i++) {
					MetaCard mc = cs.getMetaCard(i);
					if (mc == null) {
						if (i < 10000) {i = 9999;} 
						else if (i < 20000) {i = 19999;} 
						continue;
					}
					// Ignore duplicate readings.
					String searchtext = mc.getQuestion().substring(0,1);
					if (searchtext.contains(kanji)) {
						String s1 = mc.getAnswer();
						String s2 = yomi.substring(0,yomi.indexOf(" "));
						if ( mc.getID() >= 20000 ) { // just one kunyomi per moji.
							System.out.print("Ignored metacard "+i+" ("+mc.getQuestion()+"  "+mc.getAnswer()+") which already has a kun-yomi.\n");
							skippedosw.write(compound+blanks+yomi+blanks+compoundyomi+"\n");
							opcomplete = true;
							k_ignored++;
							founddupe = true;
							break;
						}
					}
				}
			} else { // Search for duplicate onyomi readings.
				findme = kanji + blanks;
				orfindme = "TBD:" + kanji;
				System.out.print("Searching for \""+kanji+"."+compound+"."+yomi+"."+compoundyomi+"\"... " );
				for (i = 1; i < cs.getNextAvailableID(); i++) {
					MetaCard mc = cs.getMetaCard(i);
					if (mc == null) {
						if (i < 10000) {i = 9999;} 
						else if (i < 20000) {i = 19999;} 
						continue;
					}
					// Ignore duplicate readings.
					String searchtext = mc.getQuestion()+mc.getAnswer();
					if (searchtext.substring(0, findme.length()).equalsIgnoreCase(findme)||
							searchtext.substring(0, orfindme.length()).equalsIgnoreCase(orfindme)) {
						String s1 = mc.getAnswer();
						String s2 = yomi+(blanks.charAt(0));
						if ( s1.substring(0, s2.length()).equalsIgnoreCase(s2) ) {
							System.out.print("Ignored metacard "+i+" ("+mc.getQuestion()+"  "+mc.getAnswer()+") with duplicate reading "+yomi+".\n");
							skippedosw.write("skipped"+kanji+"."+compound+"."+yomi+"."+compoundyomi+"\n");
							opcomplete = true;
							k_ignored++;
							founddupe = true;
							break;
						}
					}
				}
			}
			if (founddupe) { continue; }
			
			// No duplicate readings found.  Search for suitable replacement or
			// insertion point:
			for (i = 1; i < cs.getNextAvailableID(); i++) {
				int xrefID = 0;
				MetaCard mc = cs.getMetaCard(i);
				if (mc == null) {
					if (i < 10000) {i = 10000;} 
					if (i < 20000) {i = 20000;} 
					continue;
				}
				String searchtext = mc.getQuestion()+mc.getAnswer();
				boolean foundme = false;
				findme = kanji + blanks;
				orfindme = "TBD:" + kanji;
				if (searchtext.substring(0, findme.length()).equalsIgnoreCase(findme)||
						searchtext.substring(0, orfindme.length()).equalsIgnoreCase(orfindme)) {
					// Get cross reference ID:
					xrefID = i;
					
					// Add the new reading.
					if (mc.getQuestion().contains("TBD") && !kunyomi ) {
						// This is the first reading for this kanji.  Replace
						// the TBD card.
							BasicCard newmc = 
								new BasicCard(	i,
												mcQuestion,
												mcAnswer+xrefID);
							if ( cs.replaceMetaCard(newmc) )
								System.out.print("Replaced metacard "+i+":\""+mcQuestion+"\t"+mcAnswer+"\"; ");
							else
								System.out.print("Failed to replace metacard "+i+":\""+mcQuestion+"\t"+mcAnswer+"\"; ");
							k_replaced++;
							opcomplete = true; finalID = i;
					} else {
						// This is not the first reading.  Or it's a kunyomi.
						// Add extra on-yomi or kunyomi card.
						if ( kunyomi ) { // insert kun yomi
							BasicCard newmc = 
								new BasicCard(	highestkunyomi,
												mcQuestion,
												mcAnswer+xrefID);
							if ( cs.insertMetaCardAfter(newmc, i) )
								System.out.print("Inserted metacard "+highestkunyomi+" after "+i+":\""+mcQuestion+"\t"+mcAnswer+"\"; ");
							else
								System.out.print("Failed to insert metacard "+i+":\""+mcQuestion+"\t"+mcAnswer+"\"; ");
							k_inserted++;
							highestkunyomi++;
							opcomplete = true; finalID = highestkunyomi-1; 							
						} else { // insert on yomi
							BasicCard newmc = 
								new BasicCard(	highestonyomi,
												mcQuestion,
												mcAnswer+xrefID);
							if ( cs.insertMetaCardAfter(newmc, i) )
								System.out.print("Inserted metacard "+highestonyomi+" after "+i+":\""+mcQuestion+"\t"+mcAnswer+"\"; ");
							else
								System.out.print("Failed to insert metacard "+i+":\""+mcQuestion+"\t"+mcAnswer+"\"; ");
							k_inserted++;
							highestonyomi++;
							opcomplete = true; finalID = highestonyomi-1; 
						}
					}

					// Write the new card to add to the stream:
				    osw.write( newc + finalID + newcend );
					System.out.print("Wrote out new card to add to stream, id "+finalID+".\n");
					break;
				}
			}
			if (!opcomplete) {
				System.out.print("Metacard with kanji "+kanji+" not found.\n");
				k_notfound++;
				skippedosw.write(compound+blanks+yomi+blanks+compoundyomi+"\n");
				continue;
			} 

		}		
		osw.write("\n");
		osw.close( );
		skippedosw.write("\n");
		skippedosw.close();

		
		if ( asd.getCardSet() != null ) {
			AnkiStreamData.saveTagToFile(cs.toXML(), new File(cardsetfile+".out"));
		}
		if ( asd.getStream() != null ) {
			AnkiStreamData.saveTagToFile(asd.getStreamTag(), new File(cardsetfile+".out"));
		}		
		if ( asd.getPreferences() != null ) {
			AnkiStreamData.saveTagToFile(asd.getPreferencesTag(), new File(cardsetfile+".out"));
		}		

		System.out.print("\nReplaced:"+k_replaced+"  Inserted:"+k_inserted+"  Ignored:"+k_ignored+"\n");
		System.out.print("Done.\n");
	}

	/** Converts a box to a stream. 
	 */
	public static void AnkiBoxToAnkiStream( String infile, String outfile ) throws Exception {
		
		AnkiBoxData abd = AnkiBoxData.loadDataFromFile(infile);
		AnkiStreamData asd = new AnkiStreamData();
		
		// No changes to prefs or card set.
		if (abd.getPreferences() != null ) {
			asd.setPreferences(abd.getPreferences());
		}
		if (abd.getCardSet() != null ) {
			asd.setCardSet(abd.getCardSet());
		}

		// Decide base besttimesfornextreview for each partition.
		// And convert each card and add it to a new stream.
		Stream s = new Stream( outfile );
		Box b = abd.getBox();
		s.setCardSetFile(outfile);
		s.setName(b.getName() + " Stream");
		long nextreviewtimeinseconds = 0;
		Vector v = b.getCardsInCompartment(
				b.partitionNumberToCompartmentNumber(0));
		Iterator i = v.iterator();
		while (i.hasNext()) {
			Card c = new Card((IXMLElement)i.next());
			StreamCard newc = BoxCardToStreamCard(c, nextreviewtimeinseconds);
			s.add(newc);
		}
		
		// Do prebox.
		v = b.getCardsInCompartment(1);
		System.out.print("Converting partitions to stream:\n");
		System.out.print( 0 + " seconds: " + v.size() + " cards\n" );
		i = v.iterator();
		while (i.hasNext()) {
			Card c = new Card((IXMLElement)i.next());
			StreamCard newc = BoxCardToStreamCard(c, 0);
			s.add(newc);
		}

		// Do other partitions.
		nextreviewtimeinseconds = 60;
		long nextreviewspaninseconds = 60;
		for (int p = 0; p < b.getPartitionCount(); p++  ) {
			v = b.getCardsInCompartment(
					b.partitionNumberToCompartmentNumber(p));
			System.out.print( nextreviewtimeinseconds + " seconds: " + v.size() + " cards\n" );
			i = v.iterator();
			while (i.hasNext()) {
				Card c = new Card((IXMLElement)i.next());
				StreamCard newc = BoxCardToStreamCard(c, nextreviewtimeinseconds);
				s.add(newc);
			}
			nextreviewspaninseconds *= 5;
			nextreviewtimeinseconds += nextreviewspaninseconds;
		}
		
		asd.setStream(s);
		
		if ( asd.getCardSet() != null ) {
			AnkiStreamData.saveTagToFile(asd.getCardSetTag(), new File(outfile));
		}
		AnkiStreamData.saveTagToFile(asd.getStreamTag(), new File(outfile));
		if ( asd.getPreferences() != null ) {
			AnkiStreamData.saveTagToFile(asd.getPreferencesTag(), new File(outfile));
		}
		
	} 
	
	/** Converts an Anki Box style flash card to an Anki Stream style card. 
	 */
	public static StreamCard BoxCardToStreamCard( Card c, long nextreviewtimeinseconds ) 
	throws Exception
	{

		StreamCard sc = new StreamCard();
		Date lastReviewDate = new Date(new Date().getTime() - 240000);
		sc.setID( c.getID() );
		sc.setCreationDate(BoxCardTimeToStreamCardTime(c.getCreationDate()));
		sc.setLastReviewDate(StreamCard.dateform.format(lastReviewDate));
		sc.setBestNextReviewDate(EstimateBestReviewTime(c, nextreviewtimeinseconds));
		sc.setLastHundredHistory(InventHistory(c));
		sc.setLastWrongDate(BoxCardTimeToStreamCardTime(c.getLastWrongDate()));
		sc.setLifetimeRights(c.getLifetimeRights());
		sc.setLifetimeWrongs(c.getLifetimeWrongs());
		
		// Make up an average forget period (= current time - last wrong time)
		SimpleDateFormat dateform = 
			new SimpleDateFormat( "yyyy.MM.dd hh:mm:ss aa" );
		
		String s = sc.getLastWrongDate();
		if ( s.equalsIgnoreCase( "never missed" ) ) {
			sc.setAverageForgetPeriod("never missed");
		} else {
			long lwt = (dateform.parse(s)).getTime();
			long ct = new Date().getTime();
			if ( lwt > ct ) { lwt = ct; }
			long period = ct - lwt;
			sc.setAverageForgetPeriod(StreamCard.formatTimeSpan(period));
		}		
		
		return sc;		
	}
	
	public static String InventHistory(Card c) {
		StringBuffer sb = new StringBuffer("");
		int rights = Math.min( c.getLifetimeRights(), 100 );
		int wrongs = Math.min( c.getLifetimeWrongs(), 100 );
		int total = Math.min( (rights + wrongs), 100 );
		if ( c.getStreak() > 0 ) { 
			for ( int i = 0; i < Math.abs(c.getStreak()); i++ ) {
				sb.insert(0, '1');
			}
			rights -= c.getStreak();
			while ( (rights > 0) || (wrongs > 0) ) {
				if ( wrongs > 0 ) {sb.insert(0, '0'); wrongs--;}
				if ( rights > 0 ) {sb.insert(0, '1'); rights--;}
			}
		} else {
			for ( int i = 0; i < Math.abs(c.getStreak()); i++ ) {
				sb.insert(0, '0');
			}
			wrongs += c.getStreak();
			while ( (rights > 0) || (wrongs > 0) ) {
				if ( rights > 0 ) {sb.insert(0, '1'); rights--;}
				if ( wrongs > 0 ) {sb.insert(0, '0'); wrongs--;}
			}
		}
		return sb.toString();
	}
	
	public static String EstimateBestReviewTime( Card c, long nextreviewtimeinseconds ) 
	throws Exception
	{
		
		SimpleDateFormat oldform = 
			new SimpleDateFormat( "yyyy.MM.dd hh:mm aa" );
		SimpleDateFormat newform = 
			new SimpleDateFormat( "yyyy.MM.dd hh:mm:ss aa" );
		
		long timespanmillis = nextreviewtimeinseconds*1000;
/*		
		float wrongs = (float)c.getLifetimeWrongs();
		float rights = (float)c.getLifetimeRights();
		float total = wrongs+rights;
		float dragratio = rights / total;
		float extratimefloat = timespanmillis*dragratio;
		long extratimelong = (long)extratimefloat;
		long totaltimespan = timespanmillis + extratimelong;
*/
		Date nextreviewtime = new Date(new Date().getTime() + timespanmillis);
		
		return newform.format( nextreviewtime );
	}

	public static String BoxCardTimeToStreamCardTime( String bct ) 
	throws Exception
	{
		if (bct.compareTo("never missed") == 0) return bct;
		if (bct.compareTo("unreviewed") == 0) return bct;

		SimpleDateFormat oldform = 
			new SimpleDateFormat( "yyyy.MM.dd hh:mm aa" );
		SimpleDateFormat newform = 
			new SimpleDateFormat( "yyyy.MM.dd hh:mm:ss aa" );

		return newform.format( oldform.parse(bct) );
	}
	
	public static void RemoveDuplicates( String infile, String outfile ) {
		AnkiStreamData asdin = AnkiStreamData.loadDataFromFile( infile );
		CardSet cs = asdin.getCardSet();
		Stream stream = asdin.getStream();
		boolean done = false;
		int dupes = 0;
		while ( !done ) {
			Iterator i = stream.iterator();
			Vector ids = new Vector();
			while (i.hasNext()) {
				StreamCard sc = (StreamCard)i.next();
				Integer id = new Integer(sc.getID());
				if (ids.contains(id)) {
					stream.remove(sc.getID());
					dupes++;
					String s = "Found and removed duplicate ";
					s+=cs.getMetaCard(sc.getID()).getQuestion();
					System.out.println(s);
					break;
				}
				ids.add(id);
			}
			if (!(i.hasNext())) {done = true;}
		}
		
		AnkiStreamData asd = new AnkiStreamData();
		asd.setCardSet(cs);
		asd.setStream(stream);
		AnkiStreamData.saveTagToFile(asd.getCardSetTag(), new File(outfile));
		AnkiStreamData.saveTagToFile(asd.getStreamTag(), new File(outfile));
		
		System.out.println("\nTotal duplicates: " + dupes );		
	}
	
	public static void HBXToStream( String infile, String outfile ) throws IOException {
		
		initializestreams( infile, outfile );
		writeheader();
		
		Stream stream = new Stream();
		stream.setName(infile);
		stream.setCardSetFile(outfile);
		stream.setDefaultReviewLength(3600000);
		stream.setAnswerFontSize(24);
		stream.setQuestionFontSize(28);
		stream.setSecondsPerCard(15);
		
		CardSet cs = new CardSet();
		cs.setName(infile);
		
	    // Pull out all cards and make stream cards from them.
		String line = br.readLine();
		long currenttime = new Date().getTime();
		long reviewtime = new Date().getTime() + 86400000;
		long reviewspan = 86400000;
		while (!(line == null)) {
			while ((line != null) && 
					!(line.equalsIgnoreCase("<prebox>")) &&
					!(line.equalsIgnoreCase("<postbox>"))) { 
				line = br.readLine(); 
				if (line.contains("</box>")) {break;}
			}
			if (line.contains("</box>")) {break;}
			if (line != null) { 
				line = br.readLine();
				String cardids[] = line.split(" ");
				for (int i = 0; i < cardids.length; i++ ) {
					StreamCard sc = new StreamCard();
					sc.setID(Integer.parseInt(cardids[i]));
					sc.setAverageForgetPeriod("never missed");
					String bnrd = StreamCard.formatTime(reviewtime);
					sc.setBestNextReviewDate(bnrd);
					sc.setCreationDate(StreamCard.getTimeString());
					sc.setLastReviewDate(StreamCard.formatTime(currenttime-reviewspan));
					sc.setLastHundredHistory("1");
					sc.setLastWrongDate("never missed");
					sc.setLifetimeRights(1);
					sc.setLifetimeWrongs(0);
					stream.add(sc);
				}
			}
			reviewtime += 432000000;
			reviewspan += 432000000;
		}
		
		// Convert all metacards
		while (!(line == null)) {
			while ((line != null) && !(line.contains("<cardset"))) { 
				line = br.readLine(); 
			}
			if (line !=null) {
				line = br.readLine();
				// make a new metacard from each line.
				while ( !(line.contains("</cardset>")) ) {
					String[] elements = line.split("\t");
					String q = elements[1];
					String a = elements[3] + "    " + elements[2];
					MetaCard mc = 
						new BasicCard(Integer.parseInt(elements[0]), q, a);
					cs.addMetaCard(mc);
					line = br.readLine();
				}
			}
		}
		
		AnkiStreamData asd = new AnkiStreamData();
		asd.setCardSet(cs);
		asd.setStream(stream);
		AnkiStreamData.saveTagToFile(asd.getCardSetTag(), new File(outfile));
		AnkiStreamData.saveTagToFile(asd.getStreamTag(), new File(outfile));
						
	}
	
	public static void scrambleWordlist( String infile, String outfile ) throws IOException {

		Random r = new Random( new Date().getTime() );
		
		initializestreams(infile, outfile);
		
		Vector lines = new Vector();
		
		String line = br.readLine();
		while (line != null) {
			lines.add(line);
			line = br.readLine();
		}
		
		br.close();

		while (lines.size() > 0) {
			line = (String)lines.remove(Math.abs(r.nextInt() % lines.size()));
			osw.write(line+"\r\n");
		}
			
		osw.close();
	}
	
	public static String wordListLineToAnkiStreamLine( String in ) {
		
		int firstslash = in.indexOf("/");
		
		if ( firstslash == -1 ) return "";
		
		String answer = in.substring(0, firstslash);
		String question = in.substring(firstslash,in.length()-firstslash);

		question.replaceAll(",", "\\,");
		answer.replaceAll(",", "\\,");
		
		return "";
	}
	
	public static void initializestreams( String infile, String outfile ) throws IOException {
		// Initialize Streams.
		try {
			FileInputStream fis = new FileInputStream( infile );
			isr = new InputStreamReader( fis, "UNICODE" );
		} catch ( Exception e ) {
			return;
		}
		try {
			FileOutputStream fos = new FileOutputStream( outfile );
			osw = new OutputStreamWriter( fos, "UNICODE" );
		} catch ( Exception e ) {
			return;
		}
		StringBuffer sb = new StringBuffer();
		int c;
		while ( true ) {
			c = isr.read();
			if ( c == -1 ) break;
			sb.append( (char)c );
		}
		sr = new StringReader( sb.toString() );
		br = new BufferedReader( sr );
	}
	
	public static String makesafe( String s ) {
		s = s.replaceAll( "&", "and" );
		return s;
	}
	
	public static void writeheader() throws IOException {
		// The XML header
		osw.write( "<?xml version=\"1.0\"?>\r\n\r\n" );
		osw.write(
			"<!DOCTYPE ankistreamdata SYSTEM \"ankistreamdata.dtd\">");
		osw.write( "\r\n\r\n" );
	    osw.write( "<ankiboxdata>\r\n" );
	    osw.write( "<cardset name=\"Generated Card Set\">\r\n" );		
	}
	
	public static void writestring( String s ) throws IOException {
		osw.write( s );
	}

	public static void writemetacard( int id, String front, String back ) throws IOException {
		osw.write( "<metacard id=\"" + id + "\" type=\"basic\">" + makesafe(front) + "\t" + makesafe(back) + "</metacard>\r\n" );
	}
	
	public static void writefooter() throws IOException {
	    osw.write( "</cardset>\r\n" );
	    osw.write( "</ankistreamdata>\r\n" );
		osw.close( );		
	}
	
	public static void main( String[] args ) throws Exception {
		scrambleWordlist("wordlists/level_1_prescramble","wordlists/level_1_left");
		//AddReadings( "newreadings.txt", "RTKIIStream.xml", "newstreamcards.txt" );
		//HBXToStream("kaado.hbx", "RTKStream.xml");
		//RemoveDuplicates( "RTKStream.xml", "RTKStream.xml.out" );
	}
		
} // end class Converters
