package ankistream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.Random;
import java.util.Iterator;
import java.awt.Toolkit;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane; 
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.Timer;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** An dialog for exporting cards from the current card set and ccontainer.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class ExportCardsDialog extends JDialog {

	// The preferences settings indicated on this form.
	AnkiStreamFrame owner;
	
	JRadioButton exportminbutton;
	JRadioButton exportallbutton;

	/** Creates and shows a new export cards dialog using the specified
	  * preferences.
	 */
	public ExportCardsDialog( AnkiStreamFrame newowner ) {
		
		// Set the owner.
		super( newowner );
		
		owner = newowner;

		// Layout Controls.
		getContentPane().setLayout( 
				new BoxLayout(getContentPane(), BoxLayout.Y_AXIS) );

		// Instructions labels.
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.setBorder( BorderFactory.createTitledBorder( "" ) );
		String s=Strings.get(188);
		String s2=Strings.get(189);
		JPanel p2 = new JPanel(new FlowLayout(FlowLayout.LEFT,1,1));
		p2.add(new JLabel(s));
		p.add(p2);
		p2 = new JPanel(new FlowLayout(FlowLayout.LEFT,1,1));
		p2.add(new JLabel(s2));
		p.add(p2);
		add(p);
		
		// Export options button group.
		p = new JPanel();
		p.setBorder( BorderFactory.createTitledBorder( "" ) );
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p2 = new JPanel( new FlowLayout(FlowLayout.CENTER,1,1) );
		ButtonGroup bg = new ButtonGroup();
		exportminbutton = new JRadioButton( Strings.get(190) );
		bg.add( exportminbutton );
		p2.add( exportminbutton );
		p.add(p2);
		p2 = new JPanel( new FlowLayout(FlowLayout.CENTER,1,1) );
		exportallbutton = new JRadioButton( Strings.get(191) );
		bg.add(exportallbutton);
		p2.add( exportallbutton );
		p.add(p2);
		exportminbutton.setSelected( true );
		add(p);
		
		// Export buttons.
		p = new JPanel( new FlowLayout(FlowLayout.CENTER,1,1) );
		p.setBorder( BorderFactory.createTitledBorder( "" ) );
		JButton b = new JButton( Strings.get(192) );
		b.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { 
				onExport();				
				dispose( );		
			}
		} );
		p.add( b );
		p.add( new JLabel(" ") );
		b = new JButton( Strings.get(73) );
		b.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { dispose( ); }
		} );
		p.add( b );
		add( p );
		
		setTitle( Strings.get(193) );
		pack();
		setLocationRelativeTo( owner );
		setModal( true );
		setVisible( true );
	}	
	
	public void onExport() {

		JFileChooser jfc = new JFileChooser( );

		int result = jfc.showSaveDialog( this );

		if ( result == JFileChooser.APPROVE_OPTION ) {

			File exportfile = jfc.getSelectedFile( );

			FileOutputStream fos;
			OutputStreamWriter osw;
			try {
				fos = new FileOutputStream( exportfile );
				osw = new OutputStreamWriter( fos, "UNICODE" );
			} catch ( Exception e ) {
				return;
			}
			
			Stream s = (Stream)(owner.getData().getCardContainer());
			CardSet cs = (CardSet)(owner.getData().getCardSet());
			Iterator i = s.iterator();
			
			// Show the wait dialog if necessary.
			int totalc = s.getCardCount();
			if ( totalc > 100 ) {
				owner.setWaitDialogVisible(true, Strings.get(194));
				setVisible(false);
			}
			
			// Export each card.
			int exported = 0;
			while (i.hasNext()) {
				StreamCard sc = (StreamCard)i.next();
				String line = "";
				MetaCard mc = cs.getMetaCard(sc.getID());
				//Question
				line += escapeCommas(mc.getQuestion()) + ",";
				//Answer
				line += escapeCommas(mc.getAnswer());
				if (exportallbutton.isSelected()) {
					line += ",";
					//creationdate="2008.08.05 07:58:49 PM"
					line += sc.getCreationDate() + ",";
					//lastreviewdate="2008.08.08 05:45:22 PM"
					line += sc.getLastReviewDate() + ",";
					//lastreviewspan="10 days, 20 hours, 25 minutes, 0 seconds"
					line += escapeCommas(sc.getLastReviewSpan()) + ",";
					//bestnextreviewdate="2008.08.19 02:21:44 PM"
					line += sc.getBestNextReviewDate() + ",";
					//lasthundredhistory="11111"
					line += sc.getLastHundredHistory() + ",";
					//lastwrongdate="never missed" 
					line += sc.getLastWrongDate() + ",";
					//lifetimerights="5" 
					line += sc.getLifetimeRights() + ",";
					//lifetimewrongs="0" 
					line += sc.getLifetimeWrongs() + ",";
					//averageforgetperiod="never missed"
					line += escapeCommas(sc.getAverageForgetPeriod());
				}
				line += "\r\n";
				try { osw.write(line); }
				catch (Exception e) {
					owner.setMessage( Strings.get(199)+exportfile.getName()
							+Strings.get(200));
					owner.setWaitDialogVisible(false);
					return;
				}
				exported++;
				owner.setMessage(Strings.get(196)+exported+Strings.get(197)+
						exportfile.getName()+Strings.get(198));
				}
			
			owner.setWaitDialogVisible(false);
			
			try { osw.close(); }
			catch (Exception e) { 
				owner.setMessage( Strings.get(199)+exportfile.getName()+
						Strings.get(200));
				return;
			}
		} 
	}
	
	// Changes all "," to "\,"
	String escapeCommas( String s ) {
		return s.replace(",", "\\,");
	}
}
