package ankistream;

import java.awt.Toolkit;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Arrays;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.UIManager;
import javax.swing.UIDefaults;
import javax.swing.plaf.FontUIResource;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane; 
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** Searches for and provides unicode fonts if available.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public final class FontController {
	
	/** Searches the system for unicode-enabled fonts and returns either the
	  * one most tested with AnkiBox, or the first one on the list.
	 */ 
	public static Font getUnicodeFont( ) {
		// Based on comp.lang.java.programmer post by Paul Tomblin.
		Font[] allfonts = 
			GraphicsEnvironment.getLocalGraphicsEnvironment().getAllFonts(); 
		Vector goodfonts = new Vector();
		int fontcount = 0; 
		// From the web: "4E00 - 9FFF: CJK Unified Ideographs"
		String sample = "\u4e00"; 
		for (int j = 0; j < allfonts.length; j++) { 
			if (allfonts[j].canDisplayUpTo(sample) == -1 ) {
					goodfonts.add( allfonts[j] );
			}
		}
		if ( goodfonts.size() > 0 ) {
			// Try to find a tested font.
			Font[] testedFonts = new Font[] { 
				new Font( "Dialog.plain", Font.PLAIN, 16 ),
				new Font( "MS UI Gothic", Font.PLAIN, 16 ),
				new Font( "Serif.plain", Font.PLAIN, 15 ),
				new Font( "SansSerif.plain", Font.PLAIN, 15 ),
			};
			Iterator i = Arrays.asList( testedFonts ).iterator();
			Font f;
			while ( i.hasNext() ) {
				f = (Font)i.next();
				if ( containsMatching( goodfonts, f.getName() ) )
					return f;
			}
			// No tested font found. Just use first unicode font instead.
			return (Font)goodfonts.get( 0 ); 
		}
		else
			return allfonts[0]; // No unicode font found.  Use first font.
	}
	
	/** Returns true if the given vector of fonts contains a font with the
	  * given name.
	 */
	public static boolean containsMatching( Vector v, String fontname ) {
		Iterator i = v.iterator();
		Font f;
		while ( i.hasNext() ) {
			f = (Font)i.next();
			if ( f.getName().compareToIgnoreCase( fontname ) == 0 ) 
				return true;
		}
		return false;
	}
	
	public static Font getSizedFont( float size ) {
		return getUnicodeFont().deriveFont( size );
	}
	
	public static Font getBigFont( ) {
		return getSizedFont( (float)(getUnicodeFont().getSize() * 3.0) );
	}
	
	/* Sets all default UI fonts to the given font.
	 */
	public static void setDefaultFonts( Font targetfont ) {
		UIDefaults defaults = UIManager.getDefaults( ); 
		Enumeration keys = defaults.keys( ); 
		while(keys.hasMoreElements()) { 
			Object key = keys.nextElement( ); 
			Object value = defaults.get( key ); 
			if( value != null && value instanceof Font ) { 
				UIManager.put( key, null ); 
				Font font = UIManager.getFont( key ); 
				if( font != null ) { 
					float size = font.getSize2D( );
					UIManager.put( key, new FontUIResource( targetfont ) ); 
					} 
				} 
			} 
		}
}
