package ankistream;

import java.awt.Toolkit;
import java.awt.Container;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;
import java.util.TreeSet;
import java.util.Collections;
import java.util.Random;
import java.util.NoSuchElementException;
import java.util.zip.DataFormatException;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JTextArea;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** A type of metacard that holds display information for a Kanji flash card as
  * specified in James Heisig's "Remembering the Kanji" vol. 1.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class HeisigCard extends MetaCard {
	
	private String heisigid;	// Number of the card in the heisig books.
	private String keyword;	// Keyword for kanji meaning.
	private String kanji;		// The kanji.
	private String story;		// Mnemonic story for memorizing the kanji.

	/** Gets a short string representing this card's "question" (front side)
	  * for use in card browsing tables and such.
	 */
	public String getQuestion( ) { return keyword; }

	/** Gets a short string representing this card's "answer" (back side)
	  * for use in card browsing tables and such.
	 */
	public String getAnswer( ) { return kanji; }
	
	public void initializeFromCData( ) {
		String cdata = xml.getContent();
		String[] tokens = cdata.split( "\t" );
		if ( tokens.length >= 4 ) {
			heisigid = tokens[0];
			keyword = tokens[1];			
			kanji = tokens[2];			
			story = tokens[3];			
		} else {
			heisigid = " ";
			keyword = "Card Error";			
			kanji = " ";			
			story = "Error loading flash card.";			
		}				
	}

	public MetaCardDisplayPanel getDisplayPanel( ) {
		return new hcDisplayPanel( );
	}

	public MetaCardDisplayPanel getEditPanel( ) {
		return new hcEditPanel( );
	}

	public MetaCardDisplayPanel getEditPanel( Component c ) {
		return new hcEditPanel( );
	}

	public MetaCardDisplayPanel getEditPanel( Component c, Stream s ) {
		return new hcEditPanel( );
	}

	private class hcDisplayPanel extends MetaCardDisplayPanel {
		
		protected JTextField heisigidfield;
		protected JTextField keywordfield;
		protected JTextArea storyarea;
		protected JTextField kanjifield;
		protected JButton savecardtocardsetbutton;
				
		public hcDisplayPanel( ) {

			
		} // end hcDisplayPanel constructor

		public void setQuestionFontSize( float newVal ) { }
		
		public void setAnswerFontSize( float newVal ) { }
		
		public void layoutPanel( ) {
			layoutPanel( null );
		}

		public void layoutPanel( Component c ) {

			setPreferredSize( new Dimension( 320, 360 ) );
			setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
			setBorder( BorderFactory.createTitledBorder( "" ) );
		
			JPanel p = new JPanel( new FlowLayout( ) );
			p.add( new JLabel( "Card ID: " ) );

			heisigidfield = new JTextField( );
			heisigidfield.setPreferredSize( new Dimension( 50, 20 ) );
			p.add( heisigidfield );

			p.add( new JLabel( "  Keyword: " ) );
			keywordfield = new JTextField( keyword, 8 );
			keywordfield.setPreferredSize( new Dimension( 300, 20 ) );
			p.add( keywordfield );
			add( p );

			p = new JPanel( new FlowLayout() );
			kanjifield = new JTextField( );
			kanjifield.setBorder( BorderFactory.createTitledBorder( "" ) );
			kanjifield.setFont( FontController.getBigFont( ) );
			kanjifield.setPreferredSize( new Dimension( 120, 120 ) );
			kanjifield.setHorizontalAlignment( JTextField.CENTER );
			p.add( kanjifield );			
			add( p );

			p = new JPanel( new FlowLayout( ) );
			p.add( new JLabel( "Story:" ) );
			add( p );
			p = new JPanel( new FlowLayout( ) );
			storyarea = new JTextArea( 9, 17 );
			storyarea.setWrapStyleWord( true );
			storyarea.setLineWrap( true );
			p.add( new JScrollPane( storyarea ) );
			add( p );
			showQuestion( );
		}

		public void showQuestion( ) {
			heisigidfield.setText( "" );
			keywordfield.setText( keyword );
			kanjifield.setText( "" );
			storyarea.setText( "" );
		}

		public void showAnswer( ) {
			heisigidfield.setText( heisigid );
			keywordfield.setText( keyword );
			kanjifield.setText( kanji );
			storyarea.setText( story );
		}
		
		public String getEditedContent( ) {
			String rval = heisigidfield.getText();
			rval += "\t" + keywordfield.getText();
			rval += "\t" + kanjifield.getText();
			rval += "\t" + storyarea.getText();
			return rval;
		}
		
	} // end class hcDisplayPanel

	private class hcEditPanel extends hcDisplayPanel {

		public hcEditPanel( ) {
			layoutPanel( );
		} // end hcDisplayPanel constructor
		
		public void layoutPanel( ) {

			setPreferredSize( new Dimension( 320, 360 ) );
			setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
			setBorder( BorderFactory.createTitledBorder( "" ) );
		
			JPanel p = new JPanel( new FlowLayout( ) );
			p.add( new JLabel( "Card ID: " ) );

			heisigidfield = new JTextField( );
			heisigidfield.setPreferredSize( new Dimension( 50, 20 ) );
			p.add( heisigidfield );

			p.add( new JLabel( "  Keyword: " ) );
			keywordfield = new JTextField( keyword, 8 );
			keywordfield.setPreferredSize( new Dimension( 300, 20 ) );
			p.add( keywordfield );
			add( p );

			p = new JPanel( new FlowLayout() );
			p.add( new JLabel( "Kanji: " ) );
			kanjifield = new JTextField( );
			kanjifield.setBorder( BorderFactory.createTitledBorder( "" ) );
			kanjifield.setFont( FontController.getBigFont( ) );
			kanjifield.setPreferredSize( new Dimension( 120, 120 ) );
			kanjifield.setHorizontalAlignment( JTextField.CENTER );
			p.add( kanjifield );			
			add( p );

			p = new JPanel( new FlowLayout( ) );
			p.add( new JLabel( "Story:" ) );
			add( p );
			p = new JPanel( new FlowLayout( ) );
			storyarea = new JTextArea( 9, 17 );
			storyarea.setWrapStyleWord( true );
			storyarea.setLineWrap( true );
			p.add( new JScrollPane( storyarea ) );
			add( p );
			showQuestion( );
		}
		
	} // end class hcEditPanel

} // end class HeisigCard
