package ankistream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.util.Iterator;
import java.util.Random;
import java.awt.Toolkit;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane; 
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.Timer;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** An dialog for importing cards to the current card set and ccontainer.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class ImportCardsDialog extends JDialog {

	JPanel p;
	
	// The preferences settings indicated on this form.
	AnkiStreamFrame owner;
	
	JRadioButton importminbutton;
	JRadioButton importallbutton;

	/** Creates and shows a new Edit Preferences dialog using the specified
	  * preferences.
	 */
	public ImportCardsDialog( AnkiStreamFrame newowner ) {
		
		// Set the owner.
		super( newowner );
		
		owner = newowner;

		// Layout Controls.
		getContentPane().setLayout( 
				new BoxLayout(getContentPane(), BoxLayout.Y_AXIS) );
		p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.setBorder( BorderFactory.createTitledBorder( "" ) );

		// Instructions labels.
		lab(Strings.get(170));
		lab("");
		lab(Strings.get(171));
		lab(Strings.get(172));
		lab(Strings.get(173));
		lab("");
		lab(Strings.get(174));
		lab("");
		lab(Strings.get(175));
		lab(Strings.get(176));
		lab(Strings.get(177));
		lab("");
		lab(Strings.get(178));
		lab(Strings.get(179));
		add(p);
		
		// Import options button group.
		p = new JPanel();
		p.setBorder( BorderFactory.createTitledBorder( "" ) );
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		JPanel p2 = new JPanel( new FlowLayout(FlowLayout.CENTER,1,1) );
		ButtonGroup bg = new ButtonGroup();
		importminbutton = new JRadioButton( Strings.get(180) );
		bg.add( importminbutton );
		p2.add( importminbutton );
		p.add(p2);
		p2 = new JPanel( new FlowLayout(FlowLayout.CENTER,1,1) );
		importallbutton = new JRadioButton( Strings.get(181) );
		bg.add(importallbutton);
		p2.add( importallbutton );
		p.add(p2);
		importminbutton.setSelected( true );
		add(p);


		// Import buttons.
		p = new JPanel( new FlowLayout(FlowLayout.CENTER,1,1) );
		p.setBorder( BorderFactory.createTitledBorder( "" ) );
		JButton b = new JButton( Strings.get(182) );
		b.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { 
				onImport();				
				dispose( );		
			}
		} );
		p.add( b );
		p.add( new JLabel(" ") );
		b = new JButton( Strings.get(73) );
		b.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { dispose( ); }
		} );
		p.add( b );
		add( p );

		
		setTitle( Strings.get(182) );
		pack();
		setLocationRelativeTo( owner );
		setModal( true );
		setVisible( true );
	}	
	
	// Shortcut for typing less.
	private void lab( String s ) {
		JPanel p2 = new JPanel(new FlowLayout(FlowLayout.LEFT,1,1));
		p2.add(new JLabel(s)); 
		p.add(p2);
	}

	public void onImport() {

		JFileChooser jfc = new JFileChooser( );

		int result = jfc.showOpenDialog( this );

		if ( result == JFileChooser.APPROVE_OPTION ) {

			// Show the wait dialog if necessary.
			File importfile = jfc.getSelectedFile( );
			long filesize = importfile.length();
			if ( filesize > 100000 ) {
				owner.setWaitDialogVisible(true, Strings.get(195));
				setVisible(false);
			}

			// Initialize input streams.
			InputStreamReader isr;
			
			try {
				FileInputStream fis = new FileInputStream( importfile );
				isr = new InputStreamReader( fis, "UNICODE" );
			} catch ( Exception e ) {
				owner.setWaitDialogVisible(false);
				return;
			}
			StringBuffer sb = new StringBuffer();
			int c;
			try {
				while ( true ) {
					c = isr.read();
					if ( c == -1 ) break;
					sb.append( (char)c );
				}
			} catch (Exception e) {
				owner.setWaitDialogVisible(false);
				return;
			}
			StringReader sr = new StringReader( sb.toString() );
			BufferedReader br = new BufferedReader( sr );
			
			Stream s = (Stream)(owner.getData().getCardContainer());
			CardSet cs = (CardSet)(owner.getData().getCardSet());
			Iterator i = s.iterator();

			// Import each card.
			BasicCard bc = null;
			StreamCard sc = null;
			int imported = 0;
			int cur_id = cs.getNextAvailableID();
			String line = null;
			try { line = br.readLine(); } catch (Exception e) {}
			while ( line != null ) {
				boolean success = false;
				line = line.replace("\\,", "|\\||");
				String[] tokens = line.split(",");
				if (tokens.length >= 2) { 
					//Question & Answer
					bc = new BasicCard(cur_id,
							restoreCommas(tokens[0]),
							tokens[1]);
					sc = new StreamCard();
					sc.setID(cur_id);
					success = true;
					if (importallbutton.isSelected()) {
						try {
							//creationdate="2008.08.05 07:58:49 PM"
							if (tokens.length>=3) 
								{sc.setCreationDate(restoreCommas(tokens[2]));}
							//lastreviewdate="2008.08.08 05:45:22 PM"
							if (tokens.length>=4) 
								{sc.setLastReviewDate(restoreCommas(tokens[3]));}
							//lastreviewspan="10 days, 20 hours, 25 minutes, 0 seconds"
							if (tokens.length>=5) 
								{sc.setLastReviewSpan(restoreCommas(tokens[4]));}
							//bestnextreviewdate="2008.08.19 02:21:44 PM"
							if (tokens.length>=6) {
								sc.setBestNextReviewDate(restoreCommas(tokens[5]));
							}
							//lasthundredhistory="11111"
							if (tokens.length>=7) {
								sc.setLastHundredHistory(restoreCommas(tokens[6]));
							}
							//lastwrongdate="never missed"
							if (tokens.length>=8) 
								{sc.setLastWrongDate(restoreCommas(tokens[7]));}
							//lifetimerights="5" 
							if (tokens.length>=9) {
								int x = Integer.parseInt(restoreCommas(tokens[8]));
								sc.setLifetimeRights(x);
							}
							//lifetimewrongs="0" 
							if (tokens.length>=10) {
								int x = Integer.parseInt(restoreCommas(tokens[9]));
								sc.setLifetimeWrongs(x);
							}
							//averageforgetperiod="never missed"
							if (tokens.length>=11) {
								sc.setAverageForgetPeriod(
										restoreCommas(tokens[10]));
							}
						} catch ( Exception e) {
							success = false;
						}
					}
				}
				if ( success ) {
					cs.addMetaCard(bc);
					s.add(sc);
					imported++;
					cur_id++;
					owner.setMessage(Strings.get(183)+imported+
							Strings.get(184)+importfile.getName()+
							Strings.get(185));
				}
				try { line = br.readLine(); } catch (Exception e) {}
			}

			owner.setWaitDialogVisible(false);

			try { br.close(); }
			catch (Exception e) { 
				owner.setMessage(
					Strings.get(186)+importfile.getName()+Strings.get(187) );
				return;
			}
		} 
	}

	// Changes all "," to "\,"
	String restoreCommas( String s ) {
		return s.replace("|\\||", ",");
	}
}
