package ankistream;

import java.awt.Dimension;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;
import java.util.TreeSet;
import java.util.Collections;
import java.util.Random;
import java.awt.Font;
import java.util.NoSuchElementException;
import java.util.zip.DataFormatException;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import java.awt.Container;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** An interface specifying methods that must be provided by a panel that
  * displays information for a meta card.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public abstract class MetaCardDisplayPanel extends JPanel {
	
	/** Sets the font size for this panel's question cards.
	 */
	public abstract void setQuestionFontSize( float newVal );
	
	/** Sets the font size for this panel's answer cards.
	 */
	public abstract void setAnswerFontSize( float newVal );
	
	/** Sets the metacard to show only the "hint" or "question" side of the
	  * flash card.
	 */
	public abstract void showQuestion();

	/** Sets the metacard to show the "answer" side of the
	  * flash card.
	 */
	public abstract void showAnswer();
	
	/** Lays out the components on the panel. */
	public abstract void layoutPanel( );

	/** Lays out the components on the panel, specifying an external component
	  * to recieve focus after the "answer" field is traversed. */
	public abstract void layoutPanel( Component newfocusafteranswer );

	/** Gets the text representing the content of this metacard as it
	  * should be saved to xml, based on the current text displayed 
	  * in the fields on this panel.  This is used for allowing the user to
	  * save changes to metacards.
	 */
	public abstract String getEditedContent( );

} // end class MetaCardDisplayPanel
