package ankistream;

import java.awt.Dimension;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Vector;
import java.util.TreeSet;
import java.util.Collections;
import java.util.Random;
import java.util.NoSuchElementException;
import java.util.zip.DataFormatException;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** A <code>ReivewSession</code> represents one flash card review session as 
  * recorded in a Box's reviewstatistics tag.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class ReviewSession {
	
	public String date;
	public int right;
	public int wrong;
	
	public ReviewSession( String newDate, String newRight, String newWrong ) {
		date = newDate;
		try {
			right = Integer.parseInt( newRight );
		} catch ( NumberFormatException nfe ) {
			right = 0;
		}
		try {
			wrong = Integer.parseInt( newWrong );
		} catch ( NumberFormatException nfe ) {
			wrong = 0;
		}
	}

	public ReviewSession( String newDate, int newRight, int newWrong ) {
		date = newDate;
		right = newRight;
		wrong = newWrong;
	}
	
	public String getDate( ) { return date; }
	public int getRight( ) { return right; }
	public int getWrong( ) { return wrong; }
	public int getTotal( ) { return (right + wrong); }

	/** Gets a string of the form "[right] / [total] ([percent right] %)" 
	  * describing the results or status of this review session so far.
	  * @return the status string.
	 */
	public String getReviewStatusString() {
		Float fval1 = Float.valueOf(getRight());
		Float fval2 = Float.valueOf(getRight() + getWrong());
		int percentCorrect = new Float 
			((fval1.floatValue() / fval2.floatValue())*100.0).intValue();
		String s =
			getRight() + " / " + (getRight() + getWrong()) 
			+ " (" +  percentCorrect + " %)";
		return s;
	}
}