package ankistream;

import java.io.File;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;
import java.util.Iterator;
import java.util.Random;
import java.awt.Toolkit;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem; 
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.XMLWriter;
import net.n3.nanoxml.XMLParserFactory;
import net.n3.nanoxml.StdXMLReader;

/** The globalization layer for allowing quick localization of all AnkiStream's
  * text to any language.  Strings displayed in the program are loaded from a
  * pound-sign (#) delimited string file using Load().  Each line's pound-sign
  * delimited tokens represent one string in the program in each available
  * language.  For example, the first token is the English version, the second
  * the Japanese.  To print a string, use Strings.setLanguage(0) (for English),
  * then Strings.get(string number).  To print the same string in Japanese, use
  * Strings.setLanguage(1), then Strings.get(string number).
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class Strings {
	
	private static Vector s = new Vector();
	private static int current_language = 0;
	
	/* Load the program's strings from the specified set of bytes. */
	public static void load( ByteArrayOutputStream baos ) {

		s.clear();

		InputStreamReader isr;
		StringBuffer sb;
		try {

			ByteArrayInputStream bais = 
				new ByteArrayInputStream(baos.toByteArray());
			isr = new InputStreamReader( bais, "UNICODE" );
			sb = new StringBuffer();
			int c;
			while ( true ) {
				c = isr.read();
				if ( c == -1 ) break;
				sb.append( (char)c );
			}

			StringReader sr = new StringReader( sb.toString() );
			BufferedReader br = new BufferedReader( sr );			

			// Parse each line.
			String line = br.readLine();
			while (line != null) { 
				s.add(line.split("#")); 
				line = br.readLine();
			}
			
			br.close();		

		} catch ( Exception e ) {
			return;
		}
	}
	
	/* Load the program's strings from the specified data file. */
	public static void load( String filename ) {

		s.clear();

		InputStreamReader isr;
		StringBuffer sb;
		try {

			FileInputStream fis = new FileInputStream( filename );
			isr = new InputStreamReader( fis, "UNICODE" );
			sb = new StringBuffer();
			int c;
			while ( true ) {
				c = isr.read();
				if ( c == -1 ) break;
				sb.append( (char)c );
			}

			StringReader sr = new StringReader( sb.toString() );
			BufferedReader br = new BufferedReader( sr );			

			// Parse each line.
			String line = br.readLine();
			while (line != null) { 
				s.add(line.split("#")); 
				line = br.readLine();
			}
			
			br.close();		

		} catch ( Exception e ) {
			return;
		}
	}
	
	/** Returns an array of strings containing the name of each available
	  * language, it's location in the array equal to its languageNumber (used
	  * by setLanguage()).
	  * Note this array is just the tokens taken from the first line of the
	  * strings file.  So the first line of the strings file should be each
	  * language's name.
	 */
	public static String[] getLanguages() {
		return (String[])s.get(0);
	}
	
	/** Gets the current language number. */
	public static int getLanguage( ) { return current_language; }
	
	/** Sets the current language to the given number.  Subsequent calls to
	  * Strings.get() will return the string in this language.
	 */
	public static void setLanguage( int languageNumber ) {
		current_language = languageNumber;
	}
	
	/** Gets the requested string in the current language. */
	public static String get( int stringNumber ) {
		try { return ((String[])s.get(stringNumber))[current_language]; }
		catch (Exception e) { 
			try {return ((String[])s.get(stringNumber))[0];} // Try English
			catch ( Exception e2 ) {}
			// Fail loudly if even English fails.
			return "STRING LOCALIZATION ERROR"; 
		}
	}
		
} // end class StringBank
