package ankistream;

import java.awt.Dimension;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.List;
import java.util.Date;
import java.util.ArrayList;
import java.util.Vector;
import java.util.TreeSet;
import java.util.Collections;
import java.util.Random;
import java.util.NoSuchElementException;
import java.util.zip.DataFormatException;
import java.io.StringReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JCheckBox;
import java.awt.Point;
import java.awt.Font;
import java.awt.Container;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/** A component of the Anki Stream GUI that is displayed between review
  * sessions when the user is not examining any particular card.
  * <br> 
  * <br> Part of the Anki Stream project, a tool for long-term flash card
  * review.  Anki Stream is distributed under the MIT/X11 free software 
  * license, a copy of which should have come with this distribution.
  * @author Bishop Wilkins
  * @version 0.1
*/ 
public class SummaryPanel extends JPanel {
	
	private AnkiStreamFrame frame;
	
	// Panel components:
	private JLabel ccontainerlabel;
	public JTextField namefield;
	private JLabel cardcountlabel;
	private JButton cardrulesbutton;
	private JButton fontbutton;
	private JLabel lifetimereviewslabel;
	private JLabel firstcardaddedlabel;
	private JLabel lengthlabel;
	private JLabel densitylabel;
	private JLabel lastreviewlabel;
	private JLabel lastreviewcorrectlabel;
	
	private boolean acceptChangeNotifications = true;
	
	public SummaryPanel( AnkiStreamFrame newframe ) {
		
		frame = newframe;
				
		// Instantiate components.
		ccontainerlabel = new JLabel( );
		ccontainerlabel.setText(" " + frame.getCardContainerType() + ":  ");
		ccontainerlabel.setFont(
				ccontainerlabel.getFont().deriveFont(Font.BOLD));
		namefield = new JTextField( 20 );
		namefield.addFocusListener( 
			new FocusAdapter( ) {
				public void focusLost( FocusEvent e ) {
					frame.onCCNameChanged();
				}			
		} );
		cardcountlabel = new JLabel( );
		JPanel fontbuttonpanel = new JPanel(new BorderLayout());
		fontbuttonpanel.setPreferredSize(new Dimension(200, 24));
		fontbutton = new JButton( Strings.get(128) );
		fontbutton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { new FontDialog(); }
		} );
		fontbutton.setPreferredSize( new Dimension (90, 20) );
		cardrulesbutton = new JButton( Strings.get(129) );
		cardrulesbutton.setPreferredSize( new Dimension (100, 20) );
		cardrulesbutton.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { 
				new CardRulesDialog(); 
			}
		} );
		fontbuttonpanel.add(cardrulesbutton, BorderLayout.CENTER);
		fontbuttonpanel.add(fontbutton, BorderLayout.EAST);
		
		//Font smallfont = FontController.getSizedFont(12);
		Font smallfont = new JLabel().getFont().deriveFont(12);

		lifetimereviewslabel = 
			new JLabel("Lifetime Reviews: 98200 / 892002 (xx %)");
		lifetimereviewslabel.setFont(smallfont);
		firstcardaddedlabel =
			new JLabel("First Card Added: MMM dd (xx.x months ago)");
		firstcardaddedlabel.setFont(smallfont);
		lengthlabel =
			new JLabel("Length: MMM dd ~ MMM dd (xx.x weeks)");
		lengthlabel.setFont(smallfont);
		densitylabel =
			new JLabel("Avg. Density: xx.x cards per hour");
		densitylabel.setFont(smallfont);
		
		lastreviewlabel = new JLabel( );
		lastreviewcorrectlabel = new JLabel( );		

		updateSummary( );

		// Lay out the panel.
		setLayout( new BorderLayout( ) );
		setBorder( BorderFactory.createTitledBorder( "" ) );
		JPanel northpanel = new JPanel( );

		northpanel.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		GridBagLayout gbl = new GridBagLayout( );
		northpanel.setLayout( gbl );
		GridBagConstraints c = new GridBagConstraints( );
		c.fill = GridBagConstraints.NONE;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.anchor = GridBagConstraints.WEST;
		addC( northpanel, new JLabel( " " ),			gbl, c, 0, 0, 4, 1 );
		addC( northpanel, ccontainerlabel,				gbl, c, 0, 1, 1, 1 );
		addC( northpanel, namefield,					gbl, c, 1, 1, 3, 1 );
		addC( northpanel, cardcountlabel,				gbl, c, 1, 2, 2, 1 );
		addC( northpanel, fontbuttonpanel,				gbl, c, 3, 2, 1, 1 );

		if (frame.getData().getCardContainer() instanceof Stream ) {
			// Stream stats:
			if (((Stream)frame.getData().getCardContainer()).
					getCardCount()>9) {
				addC( northpanel, lifetimereviewslabel,	gbl, c, 1, 3, 3, 1 );
				addC( northpanel, firstcardaddedlabel,	gbl, c, 1, 4, 3, 1 );
				addC( northpanel, lengthlabel,			gbl, c, 1, 5, 3, 1 );
				addC( northpanel, densitylabel,			gbl, c, 1, 6, 3, 1 );
			}
		} else {
			// Box stats:
			addC( northpanel, lastreviewlabel,			gbl, c, 1, 3, 2, 1 );
			addC( northpanel, lastreviewcorrectlabel,	gbl, c, 1, 4, 2, 1 );
		}
		add( northpanel, BorderLayout.NORTH );
		
		// Add listeners.
		namefield.getDocument( ).addDocumentListener(
			new DocumentListener( ) {
				public void changedUpdate( DocumentEvent e ) 
					{ onCardContainerNameChanged( ); }
				public void insertUpdate( DocumentEvent e ) 
					{ onCardContainerNameChanged( ); }
				public void removeUpdate( DocumentEvent e ) 
					{ onCardContainerNameChanged( ); }
			});
		
		acceptChangeNotifications = true;
	}

	// Utility for component placement on a GridBagLayout. 
	private void addC(	Container parent,
							Component c, GridBagLayout gbl, 
							GridBagConstraints gbc,
							int xPos, int yPos,
							int width, int height ) {
		gbc.gridx = xPos;
		gbc.gridy = yPos;
		gbc.gridwidth = width;
		gbc.gridheight = height;
		gbl.setConstraints( c, gbc );
		parent.add( c );		
	}
	
	private class FontDialog extends JDialog {
		
		private JTextField questionFontSizeField;
		private JTextField answerFontSizeField;
		
		public FontDialog( ) {
			
			// Set the owner.
			super( frame );
			
			setTitle( Strings.get(89) );

			String s = String.valueOf(
			 (int)(frame.getData().getCardContainer().getQuestionFontSize()));
			questionFontSizeField = new JTextField( s, 4 );
			s = String.valueOf(
			 (int)(frame.getData().getCardContainer().getAnswerFontSize()));
			answerFontSizeField = new JTextField( s, 4 );
			
			getContentPane().setLayout(new GridLayout(0,1));
			JPanel p = new JPanel( new FlowLayout() );
			p.add(new JLabel(Strings.get(90)));
			p.add(questionFontSizeField);
			add(p);
			p = new JPanel( new FlowLayout() );
			p.add(new JLabel(Strings.get(91)));
			p.add(answerFontSizeField);
			add(p);
			p = new JPanel( new FlowLayout() );
			JButton b = new JButton(Strings.get(84));
			b.addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ae ) { onOK(); }
			} );
			p.add(b);
			b = new JButton(Strings.get(73));
			b.addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ae ) { dispose(); }
			} );
			p.add(b);
			add(p);
			
			setModal( true );
			Dimension prefSize = new Dimension(260,140);
			setPreferredSize(prefSize);
			pack();
			int x = 
				(int)(frame.getX()+(frame.getWidth()/2)-(prefSize.width / 2));
			int y = 
				(int)(frame.getY()+(frame.getHeight()/2)-(prefSize.height / 2));
			setLocation(x,y);

			setVisible( true );	
		}
				
		private void onOK( ) {
			
			// Set new font sizes if they are valid.
			int newQSize, newASize;
			try {
				newQSize = Integer.parseInt(questionFontSizeField.getText());
				if (newQSize <= 0) { newQSize = 1; }
			} catch ( Exception e ) {
				newQSize = 
				 (int)frame.getData().getCardContainer().getQuestionFontSize();
			}
			try {
				newASize = Integer.parseInt(answerFontSizeField.getText());
				if (newQSize <= 0) { newQSize = 1; }
			} catch ( Exception e ) {
				newASize = 
				 (int)frame.getData().getCardContainer().getAnswerFontSize();
			}
			frame.getData().getCardContainer().setQuestionFontSize(newQSize);
			frame.getData().getCardContainer().setAnswerFontSize(newASize);
			frame.setCardContainerChangesMade(true);
			
			dispose( );
		}
	}

	private class CardRulesDialog extends JDialog {
		
		private JTextField minreviewtimefield;
		private JTextField increasereviewtimebyfield;
		private JCheckBox maxreviewtimecheckbox;
		private JTextField maxreviewtimefield;
		private JPanel maxreviewtimepanel;
		
		private JCheckBox penalizecheckbox;
		private JTextField worstpenaltyfield;

		private JCheckBox randomizetimescheckbox;

		public CardRulesDialog( ) {
			
			// Set the owner.
			super( frame );
			
			setTitle( Strings.get(92) );

			Stream st = (Stream)frame.getData().getCardContainer();
			
			getContentPane().setLayout(
					new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));
			
			JPanel pgroup = new JPanel(new GridLayout(0,1));
			pgroup.setBorder( BorderFactory.createTitledBorder( "" ) );
			JPanel p = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
			p.add(new JLabel(Strings.get(93)));
			minreviewtimefield = new JTextField( "5", 2 );
			minreviewtimefield.setText(String.valueOf(st.getMinReviewTime()));
			p.add(minreviewtimefield);
			p.add(new JLabel(Strings.get(94)));
			pgroup.add(p);
			
			p = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
			p.add(new JLabel(Strings.get(95)));
			increasereviewtimebyfield = new JTextField("5.0",4);
			increasereviewtimebyfield.setText(
					String.valueOf(st.getIncreaseSpanBy()));
			p.add( increasereviewtimebyfield );
			p.add(new JLabel(Strings.get(96)));
			pgroup.add(p);

			maxreviewtimecheckbox = 
				new JCheckBox(Strings.get(97));
			String s = Strings.get(99);
			maxreviewtimecheckbox.setToolTipText(s);
			maxreviewtimefield = new JTextField( "60", 4 );
			maxreviewtimecheckbox.setSelected( st.getUseMaxReviewTime() );
			maxreviewtimefield.setText(String.valueOf(st.getMaxReviewTime()));
			maxreviewtimepanel = 
				new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
			maxreviewtimepanel.add(maxreviewtimecheckbox);
			maxreviewtimepanel.add(maxreviewtimefield);
			JLabel jlbl = new JLabel(Strings.get(98));
			maxreviewtimepanel.add(jlbl);
			pgroup.add(maxreviewtimepanel);
			add( pgroup );

			
			pgroup = new JPanel(new GridLayout(0,1));
			pgroup.setBorder( BorderFactory.createTitledBorder( "" ) );
			p = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
			penalizecheckbox = new JCheckBox(Strings.get(100));
			penalizecheckbox.setSelected(st.getPenalizeHardCards());
			p.add(penalizecheckbox);
			pgroup.add(p);
			
			p = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
			p.add(new JLabel(Strings.get(101)));
			worstpenaltyfield = new JTextField("2.0",4);
			worstpenaltyfield.setText(
					String.valueOf(st.getWorstPenaltyMultiplier()));
			p.add(worstpenaltyfield);
			p.add(new JLabel(Strings.get(102)));
			pgroup.add(p);
			
			p = new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
			p.add(new JLabel(Strings.get(103)));
			pgroup.add(p);
			add(pgroup);
			
			pgroup = new JPanel(new GridLayout(0,1));
			pgroup.setBorder( BorderFactory.createTitledBorder( "" ) );
			randomizetimescheckbox = new JCheckBox(Strings.get(104));
			s = Strings.get(105);
			randomizetimescheckbox.setToolTipText(s);
			randomizetimescheckbox.setSelected(st.getRandomizeReviewTimes());
			JPanel randomizetimespanel = 
				new JPanel(new FlowLayout(FlowLayout.CENTER,0,0));
			randomizetimespanel.add(randomizetimescheckbox);
			pgroup.add(randomizetimespanel);
			add(pgroup);
			
			pgroup = new JPanel(new GridLayout(0,1));
			pgroup.setBorder( BorderFactory.createTitledBorder( "" ) );
			p = new JPanel( new FlowLayout(FlowLayout.CENTER,0,0) );
			JButton b = new JButton(Strings.get(84));
			b.addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ae ) { onOK(); }
			} );
			p.add(b);
			b = new JButton(Strings.get(73));
			b.addActionListener( new ActionListener() {
				public void actionPerformed( ActionEvent ae ) { dispose(); }
			} );
			p.add(b);
			pgroup.add(p);
			add(pgroup);
			
			setModal( true );
			pack();
			Dimension prefSize = new Dimension(getWidth(),getHeight());
			int x = 
				(int)(frame.getX()+(frame.getWidth()/2)-(prefSize.width / 2));
			int y = 
				(int)(frame.getY()+(frame.getHeight()/2)-(prefSize.height / 2));
			setLocation(x,y);

			setVisible( true );	
		}
				
		private void onOK( ) {
			
			// Set new card values if they are valid.
			Stream s = (Stream)frame.getData().getCardContainer();
			boolean changed = false;

			int oldival = s.getMinReviewTime();
			int newival = -1;
			try { newival = Integer.parseInt(minreviewtimefield.getText()); }
			catch ( Exception e ) { newival = -1; }
			if ( newival < 1 || newival > 525600 ) { // 1 minute ~ 1 year
				newival = oldival;
			} 
			s.setMinReviewTime(newival);
			if (s.getMinReviewTime() != oldival) { changed = true; }
			
			float oldfval = s.getIncreaseSpanBy();
			float newfval = (float)-1.0;
			try { newfval = Float.parseFloat(
					increasereviewtimebyfield.getText()); }
			catch ( Exception e ) { newfval = -1; }
			s.setIncreaseSpanBy(newfval);
			if (s.getIncreaseSpanBy() != oldfval) { changed = true; }
			
			// Use max review time.
			boolean oldbval = s.getUseMaxReviewTime();
			s.setUseMaxReviewTime(maxreviewtimecheckbox.isSelected());
			if (s.getUseMaxReviewTime() != oldbval) { changed = true; }

			// Max review time
			oldival = s.getMaxReviewTime();
			newival = -1;
			try { newival = Integer.parseInt(maxreviewtimefield.getText()); }
			catch ( Exception e ) { newival = -1; }
			if ( newival < 1 || newival > 365000 ) { // 1 day ~ 10000 years
				newival = oldival;
			} 
			s.setMaxReviewTime(newival);
			if (s.getMaxReviewTime() != oldival) { changed = true; }

			
			// Penalties for hard cards.
			oldbval = s.getPenalizeHardCards();
			s.setPenalizeHardCards(penalizecheckbox.isSelected());
			if (s.getPenalizeHardCards() != oldbval) { changed = true; }

			oldfval = s.getWorstPenaltyMultiplier();
			newfval = (float)-1.0;
			try { newfval = Float.parseFloat(worstpenaltyfield.getText()); }
			catch ( Exception e ) { newfval = -1; }
			s.setWorstPenaltyMultiplier(newfval);
			if (s.getWorstPenaltyMultiplier() != oldfval) { changed = true; }
			
			
			// Randomize times value.
			oldbval = s.getRandomizeReviewTimes();
			s.setRandomizeReviewTimes(randomizetimescheckbox.isSelected());
			if (s.getRandomizeReviewTimes() != oldbval) { changed = true; }
			
			if ( changed ) { 
				frame.setMessage("Updated card rules.");
				frame.setCardContainerChangesMade(true); 
			}

			dispose( );
		}
	}
	
	/** Called when the user enters a change in the name field.
	 */
	public void onCardContainerNameChanged( ) {
		if (!acceptChangeNotifications) {return;}
		if ( namefield.getText( ).length( ) == 0 ) { return; }
		frame.getData( ).getCardContainer( ).setName( namefield.getText( ) );
		frame.setCardContainerChangesMade( true );
	}
		
	public void updateSummary( ) {

		// Have our DocumentListener functions ignore the notifications that
		// will fire when we update the text fields, just for this function.
		acceptChangeNotifications = false; 
		
		AnkiStreamData data = frame.getData( );
		ccontainerlabel.setText(" " + frame.getCardContainerType() + ":  ");
		namefield.setText( data.getCardContainer( ).getName( ) );
		cardcountlabel.setText(
			"(" + String.valueOf(data.getCardContainer().getCardCount()) 
			+ Strings.get(106) );
		fontbutton.setText( Strings.get(128) );
		cardrulesbutton.setText( Strings.get(129) );

		if (data.getCardContainer() instanceof Stream) {

			SimpleDateFormat dfy = new SimpleDateFormat( "yyyy" );
			SimpleDateFormat dfym = new SimpleDateFormat( "MMM dd yyyy" );
			SimpleDateFormat dfm = new SimpleDateFormat( "MMM dd" );
			Date now = new Date();

			Stream stream = (Stream)data.getCardContainer();

		    // Lifetime Reviews: 98200 / 892002 (xx %)
			String s = Strings.get(107) + stream.getLifetimeRights() + 
						" / " + stream.getLifetimeReviews();
			Float fval1 = Float.valueOf(stream.getLifetimeRights());
			Float fval2 = Float.valueOf(stream.getLifetimeReviews());
			int percentCorrect = new Float 
				((fval1.floatValue() / fval2.floatValue())*100.0).intValue();
			s += " (" + percentCorrect + " %)";
			lifetimereviewslabel.setText(s);
			
		    // First Card Added: MM dd (xx.x months ago)
			long firstcardtime = stream.getFirstCardDate();
			if ( firstcardtime == -1 ) {
				s = Strings.get(108);
			} else {
				Date fcdate = new Date(firstcardtime);
				long span = now.getTime() - fcdate.getTime();
				s = Strings.get(109);
				if ( dfy.format(now).equalsIgnoreCase(dfy.format(fcdate))) {
					// Same year.
					s += dfm.format(fcdate);
				} else { // Different year.
					s += dfym.format(fcdate);
				}
				s += " ("+StreamCard.formatTimeSpanShortForm(span)+
					Strings.get(110);
			}
			firstcardaddedlabel.setText(s);

		    // Length: MM dd ~ MM dd (xx.x weeks)
			s = Strings.get(111);
			long cclong = stream.getFrontCardBestReviewDate();
			long bclong = stream.getBackCardBestReviewDate();
			Date cctime;
			Date bctime;
			long span = -1;
			if ( (cclong != -1) && (bclong != -1) ) {
				cctime = new Date(cclong);
				bctime = new Date(bclong);
				span = bclong - cclong;
				if ( dfy.format(now).equalsIgnoreCase(dfy.format(cctime))) 
					{s += dfm.format(cctime);} else {s += dfym.format(cctime);}
				s += " ~ ";
				if ( dfy.format(now).equalsIgnoreCase(dfy.format(bctime))) 
				{s += dfm.format(bctime);} else {s += dfym.format(bctime);}
				s += " (" + StreamCard.formatTimeSpanShortForm(span) + ")";
			}
			lengthlabel.setText(s);			

		    // Avg. Density: xx.x cards per hour
			s = Strings.get(112);
			if ( span != -1 ) {
				float totalhours = span / 3600000;
				if ( totalhours < 1 ) {totalhours = 1;}
				float cardstohours = (float)stream.getCardCount()/totalhours;
				
				DecimalFormat df = new DecimalFormat("#0.0");
				df.setMaximumFractionDigits(1);

				s += Strings.get(113) + df.format(cardstohours) + 
					Strings.get(114);
			}
			densitylabel.setText(s);

		} else { // This is a box.
			ReviewSession rs = data.getBox( ).getLastReviewSession( );
			if ( rs == null ) {
				lastreviewlabel.setText( "No review sessions complete." );
				lastreviewcorrectlabel.setText( "" );
			} else {
				lastreviewlabel.setText( "Last Reviewed " + rs.getDate( ) );
				lastreviewcorrectlabel.setText( 
					rs.getReviewStatusString() + " Correct" );
			}
		}

		acceptChangeNotifications = true;
	}

} // end class SummaryPanel
