package ankistream;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.util.Random;
import java.util.Vector;
import java.util.Iterator;
import java.util.Date;
import java.awt.Toolkit;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.JOptionPane;
import javax.swing.Timer;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.Timer;
import javax.swing.ButtonGroup;

import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

/**
 * Tool for reading from JLPT word lists. <br>
 * <br>
 * Part of the Anki Stream project, a tool for long-term flash card review. Anki
 * Stream is distributed under the MIT/X11 free software license, a copy of
 * which should have come with this distribution.
 * 
 */
public class WordListVacuum extends JFrame {

	private JLabel thissessionlabel = new JLabel("");
	private JLabel thislevellabel = new JLabel("");
	private JLabel totallabel = new JLabel("");
	private JPanel mcdpp = new JPanel( new FlowLayout( ) );
	private MetaCardDisplayPanel mcdp;
	private JButton flipbutton;
	private JButton knowitbutton;
	private JButton additbutton;
	private JButton loadbutton;
	private JButton savebutton;
	private JLabel messages= new JLabel("");
	
	private InputStreamReader isr;
	private StringReader sr;
	private BufferedReader br;
	private OutputStreamWriter osw;
	
	int cardsthissession = 0;
	int totalcardspersession = 200;
	
	Vector leftv = new Vector();
	Vector knownv = new Vector();
	Vector addv = new Vector();
	Vector addqueue = new Vector();
	
	int totallevels = 4;
	
	int currentlevel = totallevels;
	String currentline = null;
	
	boolean questionphase = true;
	
	public WordListVacuum() throws IOException {
		
		loadAll();

		getContentPane().setLayout(new BorderLayout());

		JPanel p = new JPanel( );
		p.setLayout(new BoxLayout( p, BoxLayout.Y_AXIS ));
		JPanel p2 = new JPanel( new FlowLayout() );
		p2.add( thissessionlabel );
		p.add(p2);
		p2 = new JPanel( new FlowLayout() );
		p2.add( thislevellabel );
		p.add(p2);
		p2 = new JPanel( new FlowLayout() );
		p2.add( totallabel );
		p.add(p2);
		p2 = new JPanel( new FlowLayout() );
		p2.add( totallabel );
		p.add(p2);
		getContentPane().add(p, BorderLayout.NORTH);

		getContentPane().add(mcdpp, BorderLayout.CENTER);

		p = new JPanel( );
		p.setLayout(new BoxLayout( p, BoxLayout.Y_AXIS ));
		p2 = new JPanel( new FlowLayout() );
		flipbutton = new JButton( "Flip" );
		flipbutton.addKeyListener(new WLVKeyListener());
		flipbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				onFlip();
			}
		});
		p2.add( flipbutton );
		knowitbutton = new JButton( "Known" );
		knowitbutton.addKeyListener(new WLVKeyListener());
		knowitbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				try {onKnown();} 
				catch (IOException ioe) {
					System.out.println("IOException in onKnown(): " + ioe.toString() );
				}
			}
		});
		p2.add( knowitbutton );
		additbutton = new JButton( "Add" );
		additbutton.addKeyListener(new WLVKeyListener());
		additbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				try {onAdd();} 
				catch (IOException ioe) {
					System.out.println("IOException in onAdd(): " + ioe.toString() );
				}
			}
		});
		p2.add( additbutton );
		p.add(p2);
		p2 = new JPanel( new FlowLayout() );
		p2.add( messages );
		p.add(p2);
		getContentPane().add(p, BorderLayout.SOUTH);

		// Center the frame and show.
		pack();
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setLocation((int) (screenSize.getWidth() / 2) - (getWidth() / 2),
				(int) (screenSize.getHeight() / 2) - (getHeight() / 2));
		setVisible(true);
	}

	/**
	 * Handles typical Anki Box key presses.
	 */
	private class WLVKeyListener extends KeyAdapter {

		public void keyReleased(KeyEvent ke) {

			if (questionphase) {
				switch (ke.getKeyCode()) {
				case KeyEvent.VK_F:
				case KeyEvent.VK_R:
				case KeyEvent.VK_W:
					flipbutton.doClick();
					break;
				}
			} else {
				switch (ke.getKeyCode()) {
				case KeyEvent.VK_R:
					knowitbutton.doClick();
					break;
				case KeyEvent.VK_W:
					additbutton.doClick();
					break;
				}
			}

		} // end keyReleased

	}
	
	private void loadAll() {
		
		leftv.clear();
		knownv.clear();
		addv.clear();
		
		// Load each word list file.
		currentlevel = 0;
		currentline = null;
		for (int c = 1; c <= totallevels; c++) {
			Vector v = loadVectorFrom("wordlists/level_"+c+"_left");
			leftv.add( v );
			if ( v.size() > 0 ) { 
				currentlevel = c; 
				currentline = (String)v.get(0);
			}
			v = loadVectorFrom("wordlists/level_"+c+"_known");
			knownv.add( v );
			v = loadVectorFrom("wordlists/level_"+c+"_add");
			addv.add( v );
		}

		// Load the addqueue file.
		addqueue = loadVectorFrom("wordlists/add_queue");

		updateLabels();
		nextLine();
		messages.setText("Loaded all words.");
	}

	private Vector loadVectorFrom( String filename ) {

		initializeinputstream( filename );

		Vector v = new Vector();

		try {
			String line = br.readLine();
			while ( line != null ) {
				line.trim();
				if ( line.contains("/") ) {v.add(line);}
				line = br.readLine();
			}
		} catch ( Exception e ) {
			
		}		

		return v;
	}
	
	private void saveCurrentLevel() throws IOException {

		// Save each word list file.
		int clindex = currentlevel-1;
		saveVectorTo((Vector)leftv.get(clindex), 
				"wordlists/level_"+currentlevel+"_left");
		saveVectorTo((Vector)knownv.get(clindex), 
				"wordlists/level_"+currentlevel+"_known");
		saveVectorTo((Vector)addv.get(clindex), 
				"wordlists/level_"+currentlevel+"_add");

		// Save the addqueue file.
		saveVectorTo(addqueue,"wordlists/add_queue");
	}
	
	private void saveVectorTo( Vector v, String f ) throws IOException {
		initializeoutputstream( f );
		Iterator i = v.iterator();
		while (i.hasNext()) {
			osw.write((String)i.next()+"\r\n");
		}
		osw.close();
	}
		
	private void onFlip() {
		mcdp.showAnswer();
		questionphase = false;
	}
	
	private void onKnown() throws IOException {
		((Vector)leftv.get(currentlevel-1)).remove(0);
		((Vector)knownv.get(currentlevel-1)).add(currentline);
		saveCurrentLevel();		
		messages.setText("Known.");
		nextLine();
		questionphase = true;
	}
	
	private void nextLine() {
		
		// Increment level if necessary.
		if (((Vector)leftv.get(currentlevel-1)).isEmpty()) {
			currentlevel--;
			if ( currentlevel == 0 ) {
				messages.setText("Finished all cards.");
				currentline = null;
				return;
			}
		}
		
		// Then get the next line.
		currentline = (String)((Vector)leftv.get(currentlevel-1)).get(0);

		BasicCard bc = new BasicCard(1, "ERROR", "ERROR");
		bc.setContents(lineToQuestion(currentline));
		bc.initializeFromCData();
		mcdp = bc.getDisplayPanel( );
		mcdp.setQuestionFontSize((float)45.0);
		mcdp.setAnswerFontSize((float)25.0);
		mcdpp.removeAll();
		mcdpp.add(mcdp);
		mcdp.layoutPanel();
		mcdp.showQuestion();
		updateLabels();
		pack();
		repaint();
	}
		
	private void onAdd() throws IOException {
		((Vector)leftv.get(currentlevel-1)).remove(0);
		((Vector)addv.get(currentlevel-1)).add(currentline);
		addqueue.add(lineToAnkiStreamImport(currentline));
		saveCurrentLevel();		
		messages.setText("Added.");
		cardsthissession++;
		nextLine();
		questionphase = true;
	}
	
	private String lineToQuestion(String s) {
		int slashindex = s.indexOf("/");
		String japanese = s.substring(0,slashindex);
		String english = s.substring(slashindex+1);
		
		int obindex = japanese.indexOf("[");
		int cbindex = japanese.indexOf("]");
		if ((obindex != -1) && (cbindex != -1)) {
			// It's a kanji card.  Parse out the yomi and put it in the answer.
			String yomi = japanese.substring(obindex+1, cbindex);
			japanese = japanese.substring(0, obindex);
			english = yomi + "  " + english;
		}
		
		japanese.replace("\t", "TAB");
		english.replace("\t", "TAB");
		
		return japanese + "\t" + english;
	}

	private String lineToAnkiStreamImport(String s) {
		int slashindex = s.indexOf("/");
		String japanese = s.substring(0,slashindex);
		String english = s.substring(slashindex+1);
		
		japanese = japanese.replace("\\,", "|!@#$%PREVIOUSCOMMAESCAPE)(*&^");
		japanese = japanese.replace(",", "\\,");
		japanese = japanese.replace("|!@#$%PREVIOUSCOMMAESCAPE)(*&^", "\\,");
		english = english.replace("\\,", "|!@#$%PREVIOUSCOMMAESCAPE)(*&^");
		english = english.replace(",", "\\,");
		english = english.replace("|!@#$%PREVIOUSCOMMAESCAPE)(*&^", "\\,");
				
		return english + "," + japanese;
	}

	private void updateLabels() {
		String s = "Cards in add queue: "+addqueue.size();
		thissessionlabel.setText(s);
		int clindex = currentlevel-1;
		s = "On level " + currentlevel + ": ";
		s += ((Vector)leftv.get(clindex)).size() + " left / ";
		s += ((Vector)knownv.get(clindex)).size() + " known / ";
		s += ((Vector)addv.get(clindex)).size() + " added (";
		float f1 = (float)((Vector)knownv.get(clindex)).size();
		float f2 = f1 + (float)((Vector)addv.get(clindex)).size();
		s += (int)((f1/f2)*100)+"%)";
		thislevellabel.setText(s);
		s = "Totals: ";
		s += getLinesIn(leftv) + " left / ";
		s += getLinesIn(knownv) + " known / ";
		s += getLinesIn(addv) + " added (";
		f1 = (float)getLinesIn(knownv);
		f2 = f1 + (float)getLinesIn(addv);
		s += (int)((f1/f2)*100)+"%)";
		totallabel.setText(s);
	}
	
	private int getLinesIn( Vector v ) {
		Iterator i = v.iterator();
		int l = 0;
		while ( i.hasNext() ) {
			Iterator ii = ((Vector)i.next()).iterator();
			while (ii.hasNext()) {
				l++;
				ii.next();
			}
		}
		return l;
	}

	private void initializeinputstream( String infile ) {
				
		// Initialize Streams.
		StringBuffer sb = new StringBuffer();
		try {
			FileInputStream fis = new FileInputStream( infile );
			isr = new InputStreamReader( fis, "UNICODE" );

			int c;
			while ( true ) {
				c = isr.read();
				if ( c == -1 ) break;
				sb.append( (char)c );
			}
			isr.close();

		} catch ( Exception e ) {
			System.out.println("Caught exception "+e.toString());
			return;
		}
		
		sr = new StringReader( sb.toString() );
		br = new BufferedReader( sr );
	}
		
	private void initializeoutputstream( String outfile ) {
		try {
			FileOutputStream fos = new FileOutputStream( outfile );
			osw = new OutputStreamWriter( fos, "UNICODE" );
		} catch ( Exception e ) {
			return;
		}
	}
	
	public static void main( String[] args ) throws IOException {
		
		System.setProperty("apple.laf.useScreenMenuBar", "true");
		WordListVacuum wlv = new WordListVacuum();
		wlv.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
	}

} // end class WordListVacuum
